*CMZ :          24/05/2000  12.21.02  by  Ararat A. Vardanyan
*-- Author :    Ararat Vardanyan   22/04/97
C
C=======================================================================
C
        PROGRAM ANI
c
C
C ********************  STATUS - alpha version 97-04  *****************
c  ANI is a program package for multidimensional data analysis  and
c nonparametric  statistical  inference.  The   mode   of   a   priori
c information used is nonparametric i.e. experimental data samples and
c realization of Monte Carlo programs.
c    The main problems to be solved are: the choice of the model most
c precisely  describing  the  experimental  data,  classification   of
c particles and  interaction  processes,  determination  of  intrinsic
c dimensionality  of  the   feature   space,   best   feature   subset
c determination,   correlation    analysis,    distribution    mixture
c coefficients determination, estimation, selection of best signal domain.
c     Procedure used: Nonparametric multivariate  probability  density
c estimation   (Rosenblatt   histogram   and    Parzen    window ),
c Leave-one-out-at-a-time" test, bootstrappisation of training samples,
c nonparametric regression, neural net classification,  random  search
c methods of net training.
c
c    PROGRAM SUMMARY
c   Title of program - ANI
c   Computer - IBM 3090,EC 1045, VAX, IBM PC-AT
c   Operating system - VM/CMS ,MVS, UNIX, LINUX
c   N of bits in a word - 32,16
c   Programming language used - FORTRAN 77
c   Card punching code - ASCII
c   Program  available  from Ashot Chilingarian,
c   Yerevan Physical Institute
c   Alikhanyan brothers st.2, Yerevan-36, Armenia.
c   e-mail: chili@yerphi.am
c
c    Key words -  Monte-Carlo  Statistical  Inference,  Nonparametric
c Methods, Pattern  Recognition,  Multivariate  Analysis,  Bayes  Risk
c estimation,   Probability   Density   estimation,    Classification,
c Artificial Neural Networks, Random search methods.
c     Characteristics of Physical Problems -  event-by-event  analysis
c of  multivariate EAS data, estimation  of  chemical  composition  of
c primary cosmic rays, determination of type  and  energy  of  primary
c particle, initiated EAS. Determination of best gamma-ray domain in
c multidimensional space of Cherenkov image parameters.
c Calculation of errors  of  obtain  physical quanties.
c     The adaptation of program complex  for  the  KfK  mainframe  and
c testing were done in 05 1993 in collaboration with Dr. J.Knapp  from
c IK1, KfK, Karlsruhe.
c     The adaptation of program complex  for  the  VAX (VMS)and
c testing were done in 06 1993 in collaboration with Dr. A.Heusler,
c MPI, HEIdelberg.
c
c
c     There exist  several  versions  of  ANI  package  developed  for
c different computers and operation systems, starting from 1995 the CMZ
c version of ANI code is supporting DEC, SGI, PC based UNIX platforms.
C            ANI PROGRAM versions
C       ErPHi BESM-6 VERSION - 1985.5
C       CERN IBM 3090 (VM) VERSION - 1986.6
C       FIAN PDP 11/70 1987.2
C       FIAN VAX VERSION - 1989.1
C       ERFI EC 1045 VERSION - 1989.2
C       PATCHY VERSION -  1990.6
C       DUBLIN VAX (UNIX) VERSION - 1990.7
C       KfK KARLSRUHE IBM 3090 (MVS) VERSION 1993.5
C       MPI HEIDELBERG VAX (MVS) VERSION 1993.6
c       KfK multiclass version (alpa-processor, UNIX) 1994.5
c       CERN-PAW-NOMAD version- 04 1995,
c        Yerphi SGI version - 1996
c         FZK  ALPHA-97 version.
c
c     Therefore the code is written in  standard  FORTRAN77  including
c    some routines from CERN and  SPS  program  libraries.
c     The particular task is determined by selecting appropriate  data
c     files, Key words and procedure variables in job  file,  named  b.in
c     (the description of entering data  is  givenc in one or two lines a
c     bove the parameter string).
c
c    The used data files are following:
c
C         B.TEM - FOR BAYES RISKS DOWNLOADING.
C         B.OUT - FOR OUTPUT INFORMATION.
C         LEARN.DAT - FOR NEURAL NET PARAMETERS.
C         B.SYS - FOR INFORMATION OF ERRORS IN INPUT STREAM
C
c
c    Depending on computer type different restrictions  on  possible
c    sizes of executed data files and formats are made:
c
C**   IN - MAX. # VARIABLES
C**   IMB - MAX. # TRAINING SAMPLES EVENTS (SUM OF BOTH)
C**   IMP - MAX. # CONTROL SAMPLE EVENTS
C**   IL - MAX. # OF DIFFERENT CLASSES
C**   IKCL- MAX. # NEAREST NEIGHBORS
C**   IPR - MAX. # PRICE VARIANTS
C**   MAXLEY - MAX. # OF NEURAL NET LAYERS
C**   MAXLEY - MAX. # OF NODES IN EACH LAYER,
C**   INY - MAX. # OF  REGRESSANDS.
C**    NT,NR,NTEPM,ISYS,IFORM1,IFORMW -   are   mathematical   numbers
C**    connected with data files.
C
C       THE PROGRAM CHECKS ANY MISMATCH IN SAMPLES NAMES,
C       VARIABLES USED , NUCLEI WIDTHS ETC...
C
C
C               DESCRIPTION OF DATA BASE :
C
C       EACH TRAINING SAMPLE IS READING FROM A FILE WITH THE NAME,
C       THAT REFLECTS MODEL USED ( GENERATION PARAMETERS,..., ETC)
C       THE DATA FILES ARE 'DIRECT' (OR SEQUENTAL) ACCESS FILES WITH
C       RECORD LENGTHS EQUALS TO VARIABLES TOTAL NUMBER(NTOT) +1.
C       THE FIRST RECORD IS NUMBER OF VARIABLES(NTOT), THE
C       SECOND - NUMBER OF EVENTS(M),
C       THIRD - VARIABLES NAMES - FOR EACH VARIABLE ONE CHARACTER*8
C       WORD.
C       THEN - M RECORDS (STRINGS), EACH CONSISTING OF NTOT  REAL NUMBERS
C        (ONE EVENT)  AND THE WEIGHTS OF EVENT.
C         PAW NTUPLES  ARE ALSO ALLOWED.
c
C        MAXIMAL LENTH OF DATA FILES NAMES 50 BYTES,
C
C
      PARAMETER (IN=100,IMB=100000,IL=5,IMP=100000,IKCL=11)
      PARAMETER (IPR=5,NBANM=100)
      PARAMETER (NT=13, NR=14, NTEMP=7, ISYS=8, IFORM1=9)
      PARAMETER (INY=2,IFORMW=12,NTJ=11)
      PARAMETER (MAXLEY=5,MAXNOD=500,INIPR=5)
C
C... CWN need COMMONs to communicate
      REAL          V1,V2,V3,V4,V5,V6,V7,V8,V9,V10,V11,V12,V13,V14,V15,
     +              V16,V17,V18,V19,V20,V21,v22,v23,v24,v25,v26,v27,
     +              v28,v29,v30,v31,v32,v33,v34
      COMMON /V/    V1,V2,V3,V4,V5,V6,V7,V8,V9,V10,V11,V12,V13,V14,V15,
     +              V16,V17,V18,V19,V20,V21,v22,v23,v24,v25,v26,v27,
     +              v28,v29,v30,v31,v32,v33,v34
C
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC		Common block for jetnet
cc in parameters 2 zeros had been removed to reduce the run time size
cc seem to be returned
      PARAMETER(MAXV=2000,MAXM=150000,MAXI=1000,MAXO=1000,MAXD=200)

      COMMON /JNDAT1/ MSTJN(40),PARJN(40),MSTJM(20),PARJM(20),
     &                OINJN(MAXI),OUTJN(MAXO),MXNDJM
      COMMON /JNDAT2/ TINVJN(10),IGFNJN(10),ETALJN(10),WIDLJN(10),
     &                SATMJN(10)
      COMMON/JNINT1/OJN(MAXV),AJN(MAXV),DJN(MAXV),TJN(MAXV),DTJN(MAXV),
     &              WJN(MAXM),DWJN(MAXM),NSELFJN(MAXM),NTSELFJN(MAXV),
     &              GJN(MAXM+MAXV),ODW(MAXM),ODT(MAXV),ETAV(MAXM+MAXV)
      COMMON /JNINT2/ MJN(0:10),MV0JN(11),MM0JN(11),NGJN(10),NLJN,
     &                IPOTTJN,ER1JN,ER2JN,SMJN(10),ICPONJN
      COMMON /JNINT4/ ILINONJN,NCJN,G2JN,NITJN,ERRLNJN(0:3),DERRLNJN,
     &                STEPLNJN(0:3),STEPMNJN,ERRMNJN,IEVALJN,
     &                ISUCCJN,ICURVEJN,NSCJN,GVEC2JN
      COMMON /JMINT1/ NDIMJN,ISWJN(10),NODESJN(0:MAXD+1),NBOJN
      COMMON /JNGAUS/ ISETJN,GASDEVJN
      COMMON /JNDATR/ MRJN(5),RRJN(100)


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c **************TAKE CARE THAT MP.GE.MB!!!!!!!************
c
c      DIMENSION OMEM1(2*IMB), O1(IMB*2)
       DIMENSION OMEM1(2*IMB), O1(IMP*2)
c**********aro***************
      dimension re(iny*IMB)
      dimension tontuple(IKCL*IN)
      dimension IRR(IL), dr3(il*il),dc0(il)
c
      DIMENSION A(IN*IMB),B(IN*IMB),T(IN*IMP),TB(IN*IMP),CV(IN*IMB)
      DIMENSION U(IMB),WLUM(IMP),UA(IMB),M22(IMP),m33(imp)
c
c ******* DS - "Very big" array, used only for vizualization of ******
c ******* density estimates, switched off for cases of big IMP  ******
c
      DIMENSION DS(IMP*IL*IKCL)
c      DIMENSION DS(IL*IKCL)
c      DIMENSION DS(IMP*IL*IN)
c
      DIMENSION SS1(IPR*IKCL),ss2(ipr*ikcl)
      DIMENSION PST(IL),DEN(IL,IKCL),FLI(IPR*IKCL)
      DIMENSION ITO(IL),ITOCON(IL),ITOPCA(IL),nito(il)
      DIMENSION E1M(IL),E2M(IL*IKCL),E3M(IL),DFM(IL*IKCL*IPR*IL)
      DIMENSION RDFM(IL*IKCL*IPR*IL), WIGHT(IL),wwight(il)
      DIMENSION goal(il),golold(IL),part(inipr*il)
      DIMENSION ERROB(IL,IKCL),GOB(IL*IKCL)
      DIMENSION ERR(IL*IKCL),AP(IPR*IL),RV(IN*IL),DI(IN),DETER(IL)
      DIMENSION PF(IL*IKCL),PFER(IKCL),PFERM(IKCL),PFERS(IKCL)
      DIMENSION CC1(IN),CC2(IN),adif(IN),ccnu1(in),ccnu2(in)
      DIMENSION SM(IN*IL),SMD(IN*IL),SV(IN*IN*IL),SVD(IN*IN*IL)
      DIMENSION F(IKCL),G(IL*IKCL),NUMB(IN)
c
c   ******** ARRAYS FOR BETEST *******
c
      DIMENSION TL1(IKCL),TL1M(IKCL) ,TL1D(IKCL)
      DIMENSION TL2(IKCL),TL2M(IKCL) ,TL2D(IKCL)
      DIMENSION DSM(11*IKCL) ,DSD(11*IKCL)
c
      DIMENSION R(IN*IN*IL),R2(IN*IN),R3(IN*IN*IL),X(IN*10),Y(IPR*IN)
      DIMENSION RB(NBANM),WP(IKCL)
c
c ********** "BIG ARRAYS"  used only in REGRO mode
c
c
c      DIMENSION NL(IMP*IKCL),DP(IMP*IKCL),EST(IMP),DEL(IMP)
       DIMENSION NL(IKCL),DP(IKCL),EST(ikcl),DEL(ikcl)
c
      DIMENSION AMAX(nbanm),AMIN(nbanm),DIS(IKCL),D(IKCL),AT(IMB)
      DIMENSION tmin(IN),tmax(IN),E(IN*IN),E3(IN*IN),RBHA(IN)
      DIMENSION RMAH(IN),COR(IN),E1(IN*IN), E2(IN*IN)
c
	dimension s1rdfm(il*il*ikcl),s2rdfm(il*il*ikcl)	
	dimension s2dfm(il*ikcl),s1dfm(il*ikcl)
	dimension pfm(il*ikcl), pfd(il*ikcl)	
        dimension c0(il),cc(ikcl),rr(il)
c
	dimension cclnew(inipr),cclnu(inipr),fpe(inipr),cclr(il,il)
      DIMENSION O(MAXNOD),Q(MAXNOD),NODES(MAXLEY),CCL(IPR,IL)
      DIMENSION NODESs(MAXLEY), TJPAW(MAXNOD*IN)
      DIMENSION SSS(IL),FMERIT(IL),TJ(MAXLEY*MAXNOD*MAXNOD)
      DIMENSION TJMEM(MAXLEY*MAXNOD*MAXNOD),TJMEMP(MAXLEY*MAXNOD*MAXNOD)	
      DIMENSION par(MAXLEY*MAXNOD*MAXNOD)		
c
      DIMENSION  probir(IN)
      dimension cmin(in),cmax(in),cdif(in),cmem1(in),cmem2(in)
      dimension ish(il),itot(il)
      DIMENSION NTIDR(IL)
c	
	CHARACTER*8 NUMREG(IN)
c
        CHARACTER*50  TSNAME(IL),RTSNAM,ICON,IDUM,HBNAME
        CHARACTER*50 IT,FORMAT1
	CHARACTER*8 TEK(nbanm*2),TEK3(nbanm*2),TEK4(nbanm),INOUT
	CHARACTER*8 MEN(10),TEB(4)
	CHARACTER*8 TEL(5), TE(5)
c
      CHARACTER*8 NDATA,NTIME
      CHARACTER*1 IP(IN)
      character*2 cvt
      CHARACTER*8 FF(IKCL),IDEN(11),PP(IPR*IL),TAGS(100),NPP(INIPR*IL)
      CHARACTER*50 JMODE,RJMODE,JDEN,RJDEN,ACCESS,JDIST
      CHARACTER*4 JDIFF
      CHARACTER*80 C16,C17
      CHARACTER*80 C1,C2,C3,C4,C5,C6,C7,C8,C9,C26,C27
      CHARACTER*80 C11,C14,C15,C18,C20,C21
      CHARACTER*80 c22,c25,c12,c13
      CHARACTER*80 c35,c38,c39,c40,c41,c42,c45,c46,c47,c48,c49,c50,c51
      CHARACTER*(IN*8)RTEK1,WTEK1
      CHARACTER*(IKCL*IL*8) DFF,RDFF
      CHARACTER*(IPR*IL*8) DPP
      CHARACTER*(INIPR*IL*8) NDPP
      CHARACTER*50 STATUS,PORT
C
	CHARACTER*6 search,qualit,qqualit,memory,begin,genert
	CHARACTER*6 normal,nnorma, qtype,qqtype
c
      EQUIVALENCE (WTEK1(1:1),TEK(1)(1:1))
      EQUIVALENCE (DFF(1:1),FF(1)(1:1))
      EQUIVALENCE (DPP(1:1),PP(1)(1:1))
      EQUIVALENCE (NDPP(1:1),NPP(1)(1:1))
c
      PARAMETER (LIMIT=900000)
      COMMON /PAWC/ MEMOR(LIMIT)
c	CHARACTER*8 wipp(4)
c
c ******************* SPECIAL FOR READING OF  NTUPLES ********
c
c  opmo - HROPEN mode parameter N (new) or X (variable)
c  LRECLR - Ntuple record lenth 0 or 8192 to be read from existing hbook file
c  LRECLW - Ntuple record lenth 0 or 8192 to be written into TUPW file
c
      CHARACTER*1 opmo
      DATA opmo/'X'/,LRECLR/1024/,LRECLW/1024/
c
c  ***********************************************************
c
c ******** FOR CERN TIMING ROUTINS ***************************
c
      DATA TLIM/20000./
c ************************************************************
c
      DATA TE/'  STUD  ','  KOLM  ',' MANN-W ',' R BHAT ',' R BHAT '/
      DATA TEB/'R MAHALO',' R BHATA','  R CORR','   BAYES'/
      DATA TEL/'   *>1  ','   *>2  ','   *>3  ','   *>4  ','   *>5  '/
      DATA MEN/'   1    ','   2    ','   3    ','   4    ','   5    ',
     *         '   6    ','   7    ','   8    ','   9    ','   10   '/
c      DATA wipp/'  DIST  ',' ALPHA  ','  WIDTH ',' LENGHT '/
C
      DATA IDEN/'   -5   ','   -4   ','   -3   ','   -2   ',
     1          '   -1   ','    0   ','    1   ',
     1          '    2   ','    3   ','    4   ','    5   '/
      DATA C1/'BAYSIAN CLASSIFICATION OF CONTROL SAMPLE(P*)'/
      DATA C2/'COVARIENCE MATRICE EIGENVECTORS'/
      DATA C3/'BAYES RISC FOR DIFFERENT NUCLEI WIDTHS '/
      DATA C4/'ONE-LEAVE-OUT- TEST OVER TRAINING SAMPLE'/
      DATA C5/'BAYES RISCS FOR DIFFERENT CLASSES'/
      DATA C6/'ONE DIMENSIONAL TESTS AND SIGNF. LEVELS'/
      DATA C7/'MEAN VALUES OF THE TRAINING SAMPLE VARIABLES'/
      DATA C8/'VARIANCES OF VARIABLES OF THE TRAINING SAMPLE'/
      DATA C45/' MEAN VALUES OF BAYES ERROR & PR. DISTANCES'/
      DATA C46/' MSD OF BAYES ERROR & PR. DISTANCES'/
      DATA C47/' MSD OF MEAN VALUES OF TS  VARIABLES'/
      DATA C48/' MSD OF VARIANCES OF TS VARIABLES'/
      DATA C9/'LOG-LIKELIHOOD FUNC. - exp. according to theor. models'/
      DATA C49/'LOG-LIKELIHOOD RATIO MEAN (first/second)'/
      DATA C50/'THE CONTROL(EXPERIMENTAL) SAMPLE'/
      DATA C11/'PROPORTIONS  (AVERAGED RISKS AND CLASSIFICATIONS) '/
      DATA C12/'BIAS OF RELETIVE ERROR, Mean of(E-E(est))/E' /
      DATA C13/'MSD OF RELATIVE ERROR DISTRIBUTION'/
      DATA C14/'MEAN OF ONE-LEAVE-OUT-TEST'/
      DATA C15/'PARZEN DENSITY ESTIMATES'/
      DATA C16/'VARIANCE OF ONE-LEAVE-OUT-TEST '/
      DATA C17/'CLASSIFICATION VARIANCE'/
      DATA C18/'TRAINING SAMPLE (OR TS WEIGHTS)'/
C      DATA C19/'DISCRIMINATION EFFICIENCY(SIGMA)'/
      DATA C20/'COVARIENCE MATRICE'/
      DATA C21/'MAHALO DIST & CORR.DIST = BHATA'/
C      DATA C26/'SUMMARY OF SELECTED GAMMAS'/
       data c22/'BOOTSTRAPIZATION OF CLASSIFICATION '/
       data c25/' BOOTSTRAP AVERAGE OF RECONSTRUCTED PROPORTION '/
       data c26/'MSE OF RECONSTRACTED PROPORTIONS'/
      DATA C27/'VARIANTS OF WEDGE MULTIDIMENSIONAL CUT'/
      DATA C35/'GAUSSIAN DENSITY N(0,1) VALUES'/
      DATA C38/'L1 METRIC DENSITY ESTIMATION  MEANS'/
      DATA C39/'L1 METRIC DENSITY ESTIMATION  VARIANCIES'/
      DATA C40/'L2 METRIC DENSITY ESTIMATION MEANS'/
      DATA C41/'L2 METRIC DENSITY ESTIMATION  VARIANCIES'/
      DATA C42/'VARIANCE OF PARZEN DENSITY ESTIMATES'/
      DATA C51/'SEPARABILITY INDEX'/
C
C
C
      CALL HLIMIT(LIMIT)
C
       OPEN (UNIT=NR,FILE='b.in', FORM='FORMATTED',STATUS='OLD')
       OPEN (UNIT=NT,FILE='b.out', FORM='FORMATTED',STATUS='UNKNOWN')
       OPEN (UNIT=NTEMP,FILE='b.tem',FORM='FORMATTED',STATUS='UNKNOWN')
c
       OPEN (UNIT=ISYS,FILE='b.sys',FORM='FORMATTED',STATUS='UNKNOWN')
c
      OPEN (UNIT=NTJ,FILE='learn.dat',form='formatted',STATUS='UNKNOWN')
C
C
               CALL TIMEST(TLIM)
               CALL DATIMH(NDATA,NTIME)
               CALL TIMED(TIME)
              WRITE(*,'(2X,''RUN BEGINS AT '',A8,2X,A8)')NDATA,NTIME
              WRITE(NT,'(/42X,''RUN BEGINS AT '',A8,2X,A8/)')NDATA,NTIME
c
C
      CALL GET(L,IL,NBUT,N,IN,INY,MAXLEY,NUMB,IAP,IPR,AP,
     *KCL,IKCL,F,NEI,AMIN,AMAX,
     * TSNAME,IT,ICON,IDUM,HBNAME,
     *NBAN,NBANM,ISH,ITOT,IMB,IMP,ICSH,ICTOT,
     *STATUS,JMODE,JDEN,JDIST,ACCESS,FORMAT1,PORT,GENERT,NORMAL,
     *EXPMAX,STRANG,INTDIM,DEBUG,nipr,goal,part,inipr,
     *NTUPW,NTIDW,MEMW,IFOW,NTUPR,NTIDR,NTIDC,MEMR,IFOR,
     *LEYERS,NODES,NITER,CF,SIM,SPREED,ISHIFT,SPEED,WSPEED,WIGHT,
     *SEARCH,QUALIT,QTYPE,MEMORY,BEGIN,
     *STITER,DPOINT,NDEL,NY,NUMREG,NR,NT)
c
c
               ISHCOD=0
	            DO i=1,l
		         ISHCOD=ISHCOD+ish(i)
	           END DO
c
c *************** Preparing headings for outputs ***********************
c
c
c        WRITE(NT,'(/5x,''l,iap,nipr'',5i5)')l,iap,nipr
c
            WRITE (DPP,7)(AP(I),I=1,IAP*L)
            WRITE (NDPP,7)(PART(I),I=1,NIPR*L)
c
          WRITE (DFF,7)(F(I),I=1,KCL)
          DFF(KCL*8+1:KCL*8+8)=  '    SUMM'
          DFF(KCL*8+9:KCL*8+17)= '    ADA'
c
7       FORMAT(50F8.4)
c
cc*****************************************************
c
c ******************* TEST OF BAYES ERROR UNBIASNESS **********************
c
         IF (JMODE.EQ.'BETEST') THEN
c	
	 M=ITOT(1)
	 MB=L*M
	 MP=ICTOT
c	
	  WRITE(NT,1987) JMODE,STATUS,JDEN,expmax,strang,NBUT,genert,
     *   N,ICTOT,L,MB,(f(i), i=1,kcl)
1987      FORMAT(13x,'NONPARAMETRIC DATA ANALYSIS, ANI-97 PACKAGE'//
     *2X,'JOB MODE - ',A19/
     *2X,'JOB STATUS - ',A19/
     *2X,'DENSITY ESTIMATIORN MODE: ',A12/
     *2X,'MAXIMAL EXPONENT ',E12.4/
     *2X,'STRANGE EVENTS SELECTION, DENSITY  < ',E12.4/
     *2X,'NUMBER OF REPETITIONS ',i6/
     *2X,'RANDOM GENERATOR USED ',A8/
     *2X,'NUMBER OF VARIABLES ',i4/
     *2X,'CONTROLE SAMPLE CONSISTS OF:',I8/
     *2X,i1,' CLASSES OF TRAINING SAMPLE CONSISTS OF:',I6/
     *2X,'DENSITY ESTMATION PARAMETERS:'/11f8.2/)
c
c  THE MEAN VALUES AND VARIANCES ARE READ FROM AMIN AND AMAX ARRAYS
c
          DO I=1,L
	            ITO(I)=M
	            e1m(I)=AMIN(I)
	            e2m(I)=AMAX(I)	
	            e3m(I)=0.
	  END DO
c
c CALCULATION OF MAHALANOBIS DISTANCE AND BAYS ERROR FOR THEORETICAL VALUES
c
           i1=0
	   i2=0	
	  DO K=1,L
	       DO I=1,N
	            i1=i1+1
		         SM(i1)=e1m(k)
	            DO J=1,N
		         i2=i2+1
			 if(i.eq.j) then	
	                      SV(I2)=e2m(k)
			 else
			      SV(I2)=e3m(k)
			 end if
	            END DO
	       END DO
	  END DO
c		
       WRITE(nt,'(/2x,'' THEORETICAL VALUES OF PROB. DISTANCES  FOR''/
     *  5(3f5.1/))')  (e1m(i),e2m(i),e3m(i),i=1,l)
c
           CALL BHAFAS(SV,SM,E,E1,E2,E3,RMAH(2),RMAH(1),RMAH(3),N)
	   SQRMAH=SQRT(RMAH(1))	
	   RMAH(4)=FG(-SQRMAH/2.)
	   CALL PRT(RMAH,1,4,1,C45,MEN,TEB,NT,jmode)
	
c	   	
          IF(STATUS.eq.'DENCURVE'.and.n.eq.1) THEN
                                         TMI=-5.
                                         TMA=5.
                                         TDIS=(TMA-TMI)/MP
                                         T(1)=TMI
		                          a(1)=GAUSS(TMI,0.,1.)
                       DO I=2,MP
                                         T(I)=T(I-1)+TDIS
					 a(I)=GAUSS(T(I),0.,1.)
                      END DO
c
c  *** PREPARING PAW VARIABLES NAMES FOR GAUSS MEANS AND VARIANCES
c
                     tags(2*(kcl+2)+1)='GAUSS0'
		     tags(2*(kcl+2)+2)='GAUSS1'
		     tags(2*(kcl+2)+3)='GAUSSD0'
		     tags(2*(kcl+2)+4)='GAUSSD1'
		     tags(2*(kcl+2)+5)='BAYES'
		     tags(2*(kcl+2)+7)='MAHALO'
		     tags(2*(kcl+2)+6)='INTEGRAL'
c
c
c  *** PREPARING NAMES FOR KNN ESTIMATION RESULTS TABLE
c	
                    DO J=1,2*(kcl+2),2
                         WRITE(TAGS(j), '(''PARfr'',I2)') J
                    END DO		
c		
		    DO J=2,2*(kcl+2),2		
                         WRITE(TAGS(j), '(''PARse'',I2)') J
                    END DO
c		
		    DO I=1,2*(kcl+2)
		         inde=index(tags(i),' ')
			 if (inde.le.7)  then
			      tags(i)(inde:inde)='s'
			 end if
		    END DO
c			
       call HBOOKN(ntidw*9,'TEST OF PARZ',2*(kcl+2)+7,'//ani98',
     *             MEMW,tags)
c
c *** PREPARING ARRAYS FOR CALCULATING STATISTICAL MOMENTS
c                 (FIRST AND SECOND) WITH FILL SUBROUTINE
c
c	
	  IF(MP.LE.11) then
	       call vzero(DSM,MP*L*(KCL+2))
	       call vzero(DSD,MP*L*(KCL+2))
	  END IF
c	
	 call vzero(TL1M,KCL+2)
	 call vzero(TL1D,KCL+2)
	 call vzero(TL2M,KCL+2)
	 call vzero(TL2D,KCL+2)
	 call vzero(cmem1,4)
	 call vzero(cmem2,4)	
         call vzero(SM,N*L)
	 call vzero(SV,N*N*L)
	 call vzero(SMD,N*L)
	 call vzero(SVD,N*N*L)
c	
       END IF
c       	
	 call vzero(S1RdfM,L*L*(KCL+2))
	 call vzero(S2RdfM,L*L*(KCL+2))
         call vzero(S1dfM,L*(KCL+2))
         call vzero(S2dfM,L*(KCL+2))
         call vzero(pfm,L*(KCL+2))
         call vzero(pfd,L*(KCL+2))
c	
c  *** TS AND CS EVENTS WEIGHTS ARE EQUAL!
c	
	  do i=1,mb
	       ua(i)=1./m
	  end do
	  do i=1,mp
	       wlum(i)=1./mp
	  end do
c	
	   jmode='ONE-LEAVE-OUT-'
C
       DO  III= 1,NBUT
	  CALL  BEGEN(B,L,N,M,e1m,e2m,e3m,genert)
c
          IF(STATUS.ne.'DENCURVE') THEN
	            m1=mp/5
	            m2=mp-m1
c
c
c *****CONTROL SAMPLE IS CONSTRACTED  20% 1 class, 80% second class ********
c	  	
	         CALL  BEGEN(T,1,N,M1,e1m(1),e2m(1),e3m,genert)
		 CALL  BEGEN(T(n*m1+1),1,N,M2,e1m(2),e2m(2),e3m,genert)
	  ENd IF
c
	  CALL MISR1(B,R2,R,UA,N,ITO,MB,L)
c
c CALCULATION OF MAHALANOBIS DISTANCE AND BAYS ERROR
c

           CALL BHAFAS(R,R2,E,E1,E2,E3,RMAH(2),RMAH(1),RMAH(3),N)
	       SQRMAH=SQRT(RMAH(1))	
	       RMAH(4)=FG(-SQRMAH/2.)
	       if (debug.ge.4.and.nbut.le.10.and.m.le.10) then
                         CALL PRT(B,N,M,L,C18,TEK,MEN,NT,jmode)
	                 CALL PRT(R2,N,L,1,C7,TEK,MEN,NT,jmode)
                         CALL PRT(R,N,N,L,C8,TEK,MEN,NT,jmode)
               end if	
c	
	  CALL UCOPY(R,R3,N*N*L)
	  DO I=1,L
			 NSH=(I-1)*N*N+1
			 deter(I)=1./sqrt(det(r3(NSH),N,N))
                         call RSINV(N,R(nsh),N,IF)
               if(debug.ge.4.and.nbut.le.10) then		
	            write(nt,'(2i5,2f12.4)')iii,i,deter(I),r(i)
               end if
          END DO
C
          jmode='ONE-LEAVE-OUT-'
        CALL PARZ(B,B,UA,PST,DEN,R,DI,N,ITO,MB,L,MB,KCL,m22,del,nei,
     *WLUM,AP,IAP,F,DETER,DS,RDFM,jmode
     * ,DEBUG,JDEN,NT,expmax,strang)
C
        call fill(RDFM,S1RdfM,S2RDFM,L*L*(KCL+2),iii,nbut)
c	
	  IF(STATUS.EQ.'DENCURVE') THEN	
C
C **************** N(O,1) AND N(1,1) MEANS ******************
C
	       CALL FILL(R2,SM,SMD,N*L,III,NBUT)
	       CALL FILL(R,SV,SVD,N*N*L,III,NBUT)
	       CALL FILL(RMAH,cmem1,cmem2,4,III,NBUT)
c
                    tontuple(2*(kcl+2)+1)=R2(1)
		    tontuple(2*(kcl+2)+2)=R2(2)
		    tontuple(2*(kcl+2)+3)=R(1)
		    tontuple(2*(kcl+2)+4)=R(2)
		    tontuple(2*(kcl+2)+5)=RMAH(4)
		    tontuple(2*(kcl+2)+7)=RMAH(1)
c		
                    do jj=1,2*(kcl+2),2
                         tontuple(jj)=RDFM(JJ)
                    end do
		    do jj=2,2*(kcl+2),2
                         tontuple(jj)=RDFM(JJ)
                    end do
c
          END IF
c	
          jmode='CLASSIFICATION'
c	
        CALL PARZ(B,T,UA,PST,DEN,R,DI,N,ITO,MB,L,MP,KCL,m22,del,nei,
     *   WLUM,AP,IAP,F,DETER,DS,DFM,jmode
     * ,DEBUG,JDEN,NT,expmax,strang)
C
        IF(STATUS.eq.'DENCURVE'.and.n.eq.1) THEN
c
c *** CALCULATION OF L1 AND L2 FOR DENSITY ESTIMATES
c
	       call TMSE(DS,A,MP,L,KCL+2, TDIS,TL1,Tl2,TINT)
c	
	       call fill(TL1,TL1M,TL1D,KCL+2,iii,nbut)
	       call fill(TL2,TL2M,TL2D,KCL+2,iii,nbut)
	       IF(MP.LE.11) then
		    call fill(DS,DSM,DSD,MP*L*(KCL+2),iii,nbut)
	       END IF
c
c ********** filling HBOOK *************
c		
	       tontuple(2*(kcl+2)+6)=tint
               call HFN(ntidw*9,tontuple)		
        ELSE	
C
	CALL PRFER(RDFM,DFM,R3,R2,PF,C0,CC,RR,DR3,DC0,L,KCL+2,
     *  nt,debug,IRR)
c     	  	
	  call fill(DFM,S1DFM,S2DFM,L*(KCL+2),iii,nbut)
	  CALL FILL(PF,PFM,PFD,L*(KCL+2),III,NBUT)
c
	END IF
C
        END DO
C
         CALL PRT(S1RDFM,L,KCL+2,L,C14,TEL,FF,NT,jmode)
	 CALL PRT(S2RDFM,L,KCL+2,L,c16,TEL,FF,NT,jmode)	
c
          IF(STATUS.EQ.'DENCURVE') THEN		
	       CALL PRT(SM,N,L,1,C7,TEL,MEN,NT,jmode)
	       CALL PRT(SMD,N,L,1,C47,TEL,MEN,NT,jmode)
               CALL PRT(SV,N,N,L,C8,TEL,MEN,NT,jmode)
	       CALL PRT(SVD,N,N,L,C48,TEL,MEN,NT,jmode)
	       CALL PRT(cmem1,1,4,1,C45,MEN,TEB,NT,jmode)
	       CALL PRT(cmem2,1,4,1,C46,MEN,TEB,NT,jmode)
c
               IF(N.EQ.1) THEN		
		  CALL PRT(TL1M,1,KCL+2,1,C38,TEL,FF,NT,jmode)
		  CALL PRT(TL1D,1,KCL+2,1,C39,TEL,FF,NT,jmode)
		  CALL PRT(TL2M,1,KCL+2,1,C40,TEL,FF,NT,jmode)
		  CALL PRT(TL2D,1,KCL+2,1,C41,TEL,FF,NT,jmode)
	            IF(MP.LE.11)then
                         CALL PRT(DSM,MP,L,KCL+2,C15,men,MEN,NT,jmode)
		         CALL PRT(DSD,MP,L,KCL+2,C42,men,MEN,NT,jmode)   	
	                 CALL PRT(A,MP,1,1,C35,men,MEN,NT,jmode)
	            END IF
	       END IF
c	
          ELSE
		  CALL PRT(S1DFM,L,KCL+2,1,C22,TEL,FF,NT,jmode)
		  CALL PRT(S2DFM,L,KCL+2,1,c17,TEL,FF,NT,jmode)
                  CALL PRT(PFM,L,KCL+2,1,C25,TEL,FF,NT,jmode)
	          CALL PRT(PFD,L,KCL+2,1,C26,TEL,FF,NT,jmode)		   		
c
	CALL PRFER(S1RDFM,S1DFM,R3,R2,PF,C0,CC,RR,DR3,DC0,L,KCL+2,
     *		nt,debug,IRR)
C	
		CALL PRT(PF,L,KCL+2,1,C11,TEL,FF,NT,jmode)
C
          END IF		   	
        go to 68
       END IF
c
c ******************* TRAINING SAMPLE GENERATION **********************
c
c
       IF(JMODE.NE.'FAST'.and.jmode.ne.'CLASSIFI'.and.jmode.ne.'EXP'
     *   ) THEN
c
		mbi=0
			do i=1,l
				mbi=mbi+itot(i)
			end do			
		ishi1=1

		ishi2=1
c ****************************  PREPARING FOR FREAD CALL !!! ************		
		call UZERO(ITO,1,L)
c
c **************         READING TRAINING SAMPLES     *************************
c
c********** PAW NTUPLES READING (ONLY 2 CLASS? < MAY BE NOT! *****************
c
         DO I=1,L
          IF(ACCESS.EQ.'NOMAD') THEN
c        WRITE(nt,'('' ID of ntuple and HBOOK UNIT'',2i5)')ntidr(i),IFOr
c
	     CALL  rnomad(ntidr(i),IFOr,NT,DEBUG,TSNAME(I),NBAN,
     *       N,ITOT(I),ITO(I),ISH(I),NUMB,TEK,
     *       AMIN,AMAX,U(ishi2),A(ishi1),opmo,lreclr,tek3)
c
c
          ELSE IF(ACCESS.EQ.'SEQUENTAL'.OR.ACCESS.EQ.'SIMPLE'.
     *           OR.ACCESS.EQ.'DELOG') THEN
c
c READING SEQUENTIAL FORMATTED (FORMAT CONTAINES IN FORMAT ) FILES FROM DATA BASE
c
             CALL FREAD(A(ishi1),U(ishi2),RB,TSNAME(I),NBAN,ITO(I),TEK,
     +       DEBUG,mbi,m22,m33(ishi2),n,numb,
     *       AMIN,AMAX,ITOT(I),FORMAT1,IFORM1,ISH(I),NT,ACCESS,tek3)
c
          ELSE
        	WRITE(ISYS,'('' UNKNOWN ACCESS MODE !'')')
        	STOP 36
          END IF
                    ishi1=ishi1+ito(i)*N
                    ishi2=ishi2+ito(i)
         END DO
          MB=0
	  MMAX=ITO(1)
	  DO I=1,L
	       MB=MB+ITO(I)
	       IF(MMAX.LT.ITO(I)) MMAX=ITO(I)
	  END DO
c
c ****************** CHECKING IF NO DATA SELECTED ***************
c
                    IF(MB.LE.0) THEN
     	    WRITE(ISYS,'('' NO DATA SELECTED, CHECK BOUNDARIES!'')')
                      STOP 37	
	            END IF
c
c ************************** CHECKING WEIGHTS  **************************
c
                    IF(u(1).eq.0.) then
                         WRITE(ISYS,'('' Wight equals to zero !'')')
        	         STOP 367
                    END IF
c
c ************************************************************************
c
                  CALL TURN(U,E1M,ITO,MB,L)
c
c ************************** THE TRAINING SAMPLE   ***********************
c
                    if (debug.ge.3.and.mmax.le.10) then
             CALL PRT(a,N,mmax,l,C18,TEK,MEN,NT,jmode)
             CALL PRT(u,1,mmax,l,C18,MEN,MEN,NT,jmode)
                    end if
c
c ************************************************************************
c
c
c
c ***     CALCULATION OF MAXIMAL AND MINIMAL VALUES OF TS VARIABLES
c
	        CALL AMINMA(A,tmin,tmax,adif,N,MB,NT)		
c
c ***   TRANSFORMATION FROM LOGARITHMIC SCALE TO NATURAL
c
                    if(ACCESS.eq.'DELOG') then
                      call delog(a,a,n,mb,expmax)
                    end if
c
               if(debug.ge.3) then
       write(nt,'(5x,''Minimal and Maximal values of Training Sample''/
     *  50(5x,e12.4,''<'',a10,'' <'',e12.4/))')
     * (tmin(i),tek(i),tmax(i), i=1,n)
               end if	
                     write(*,607)(i,ito(I),itot(i),i=1,l)
c                    write(*,907)(amin(i),tek3(i),amax(i),i=1,nban)
               if(debug.ge.3) then
                    write(nt,607)(i,ito(I),itot(i),i=1,l)
                    write(nt,907)(amin(i),tek3(i),amax(i),i=1,nban)
               end if	
c
c ***     CALCULATION OF NONPARAMETRIC MOMENTS OF TS VARIABLES
c ***      CHECKING FOR OUTLIERS
c
               If(debug.ge.5.) then
                    CALL OUTLER(A,N,MB,L,ITO,B,TEK,SIM,NT)
	       END IF
c
c
C************************** PCA TRANSFORMATION *********************************
C
	  if(normal.eq.'pca') then
	       do i=1,l
                  itopca(i)=0
               end do
               do i=1,mb
                  ua(i)=1./mb
               end do
c	
	       itopca(1)=mb
c	       CALL TURN(UA,E1M,ITOpca,MB,1)
	       CALL PRT(UA,1,8,L,C18,TEK,MEN,NT,jmode)
	       CALL MISR1(A,R2,R,UA,N,ITOpca,MB,1)
c	
	       NPCA=NEI
	    IF(mb.le.24) then
	       CALL PRT(R,N,N,1,C20,TEK,MEN,NT,jmode)
	       CALL PRT(a,N,8,L,C18,TEK,MEN,NT,jmode)
	       CALL PRT(UA,1,8,L,C18,TEK,MEN,NT,jmode)
	       write (NT,'(3(8e15.4))')(ua(ii),ii=1,24)
	    END IF
	       do i=1,npca
	          write(tek(i),'(''PCA'',i1)')i
	       end do
C
C          Carry out eigenreduction.
C
        CALL TRED2(N,N,R,cc1,cc2,r2)
        CALL PRT(r2,N,N,1,C20,TEK,MEN,NT,jmode)
        CALL TQL2(N,N,CC1,r3,r2,IERR)
        write(nt,'(/5x,''EIGENVALUES:'',i7/10f8.3)')
     *    IERR,(cc1(i),i=1,n)
        CALL PRT(R2,N,NPCA,1,C2,MEN,TEK,NT,jmode)
C
C          Determine projections OF TS ON EIGENVECTORS.
C
               CALL PROJX(N,MB,NPCA,A,B,r2,cc2)
               CALL UCOPY(B,A,NPCA*MB)
          IF(mb.le.24) then
               CALL PRT(a,NPCA,8,L,C18,TEK,MEN,NT,jmode)
          end IF
C	
           write(nt,'(/5x,''TRAINING SAMPLE PROJECTED TO''/
     *i6,''FACTORS from'',i5, '' INITIAL VARIABLES'')')NPCA,N
c
          end if

       END IF
c
c ****************** CONTROL SAMPLE GENERATION***********************
c	
       IF(JMODE.EQ.'CLASSIFICATION'.OR.JMODE.EQ.'REGRO'.
     *  OR.JMODE.EQ.'FAST'.OR.JMODE.EQ.'CLASSIFI'.or.jmode.eq.'EXP'.
     *  OR.JMODE.EQ.'REGRO-AD'.
     *  or.jmode.eq.'BUTSTRAP') THEN
c
             JDIFF='DIFF'
             IF(TSNAME(1).EQ.ICON.and.ish(1).eq.icsh) THEN
                JDIFF='SAME'
             END IF
             MNEWC=0
c
          IF(ACCESS.EQ.'NOMAD') THEN
c
c        WRITE(nt,'('' ID of ntuple and HBOOK UNIT'',2i5)')ntidc,IFOr
c
	     CALL RNOMAD(ntidc,IFOr,NT,DEBUG,ICON,NBAN,
     *       N,ICTOT,MNEWC,ICSH,NUMB,TEK4,
     *       AMIN,AMAX,WLUM,T,opmo,lreclr,tek3)
c
          ELSE IF(ACCESS.EQ.'SEQUENTAL'.OR.ACCESS.EQ.'SIMPLE'
     *       .OR.ACCESS.EQ.'DELOG') THEN
c	
        CALL FREAD(t,wlum,RB,Icon,NBAN,MNEWc,TEK4,DEBUG,
     *     ictot,m22,m33,N,NUMB,
     *     AMIN,AMAX,IcTOT,FORMAT1,IFORM1,IcSH,NT,ACCESS,tek3)
          ELSE
        	WRITE(ISYS,'('' UNKNOWN ACCESS MODE !'')')
        	STOP 46
c
          END IF
c
c ********************************************************************
c ****************** CHECKING IF NO DATA SELECTED ***************
c
                    IF(MNEWC.LE.0) THEN
     	    WRITE(ISYS,'('' NO CONTROL SELECTED, CHECK BOUNDARIES!'')')
                      STOP 137	
	            END IF
c
c ************************** CHECKING WEIGHTS  ***********************
c
                    If(wlum(1).eq.0.) then
                         WRITE(ISYS,'('' Control Wights -  zero !'')')
        	         STOP 237
                    END IF
c
c **************************  CHECKING NAMES *************************
c
          IF(jmode.ne.'CLASSIFI'.or.jmode.ne.'EXP') THEN
                              do i=1,n
                                   tek(i)=tek4(i)
                              end do
          else
                              do i=1,n
                 IF(TEK4(I).NE.TEK(I)) THEN
          WRITE (ISYS,'(''  TRAINING MISMATCH WITH CONTROL(EXP)''/
     1  2(5x,A50/))')TEK4(I),TEK(I)
                                        STOP 66
                  END IF
                              end do

          END IF
c
c *********************************************************************
c
             MP=MNEWC
             ITOCON(1)=MNEWC
             ITOCON(2)=0
c
             CALL AMINMA(t,cmin,cmax,cdif,N,MP,NT)
c
          CALL TURN(WLUM,E1M,ITOCON,MP,1)
c
c
c *********************** THE  CONTROL SAMPLE   ***********************
c
                    if (debug.ge.3.and.mnewc.le.10) then
             CALL PRT(T,N,mnewc,1,C50,TEK,MEN,NT,jmode)
             CALL PRT(wlum,1,mnewc,l,C50,TEK,MEN,NT,jmode)
                    end if
c
c ***************************************************************************
c
c
                    if(ACCESS.eq.'DELOG') then
            call delog(t,t,n,mp,expmax)
                    end if
c
C************************** PCA TRANSFORMATION *********************************
C
	  if(normal.eq.'pca') then	
C
C          Determine projections OF CS ON EIGENVECTORS.
C
               do i=1,npca
	          write(tek(i),'(''PCA'',i1)')i
	       end do
C
        NPCA=NEI
        CALL PROJX(N,MP,NEI,T,B,r2,cc2)
	CALL UCOPY(B,T,MP*NPCA)
           write(nt,'(/5x,''CONTROL SAMPLE PROJECTED TO''/
     *i6,''FACTORS from'',i5, '' INITIAL VARIABLES'')')NPCA,N
          end if
c
**************************** NEW NUMBER OF FEATURES  ************************
c
                 if(normal.eq.'pca') then
                         N=NPCA
                 end if
       END IF
c
C***************** BEGINING OF DATA ANALYSIS  ****************************
c
c
c____________________INTRODUCING OF PAW (CERN, 04, 95 )__________________	

c******************** OPENING OF NTUPLE ANI98 FOR WRITE  *****************
c
       if(ntupw.gt.0.and.jmode.ne.'DIMDIM'.and.jmode.ne.'CVM') then
c
	call HROPEN(ntidw,'ani98',hbname,'N',lreclw,ISTAT)
c
       end if
c
c
c************************************************************************
c
        WRITE(NT,87) JMODE,STATUS,JDEN,expmax,strang,NBUT,N,
     *   ICON,ICSH,ICTOT,MNEWC,
     1  (TSNAME(I),ISH(I),ITOT(I),ITO(I), I=1,L)
87      FORMAT(13x,'ANALYSIS AND NONPARAMETRIC INFERENCE (ANI-98)'//
     *2X,'JOB MODE - ',A19/
     *2X,'JOB STATUS - ',A19/
     *2X,'DENSITY ESTIMATIORN MODE: ',A12/
     *2X,'MAXIMAL EXPONENT ',E12.4/
     *2X,'STRANGE EVENTS SELECTION, DENSITY  < ',E12.4/
     *2X,'Number of BOOTSTRAP replicas ',i6/
     *2X,'Number of VARIABLES  ',i6/
     *2X,'CONTROLE (EXPERIMENTAL) SAMPLE:'/
     *2X,A30,' REL. COORDINATES:',2I6,' Selected:',i6/
c     *2X,'WEIGHTS FROM -'/
c     *2X,A40,' REL. COORDINATE',I8//
     *2X,'TRAINING SAMPLE:'/
     *5(2x,A30,' REL. COORDINATES:',2I6,' Selected:',i6/))
c
      if(ACCESS.EQ.'NOMAD') then
      write(nt,'(2x,''CONTROL AND TRAINING FROM PAW HISTOGRAMMS: '',
     *6I7)')NTIDC,(NTIDR(I),I=1,L)
      END IF
c
C********* EXPERIMENTAL SAMPLE MEANS AND CORRELATIONS ********************
c
        IF(JMODE.EQ.'CLASSIFICATION'.or.jmode.eq.'CLASSIFI'.or
     *.jmode.eq.'EXP'. AND.DEBUG.GE.2) THEN
c

                    if(debug.ge.3) then	
                       WRITE(NT,707) MNEWC,ICTOT
       write(nt,'(/5x,''Minimal and Maximal values of Exp. Data''/
     *  50(5x,e12.4,'' <'',a10,'' <'',e12.4/))')
     * (cmin(i),tek(i),cmax(i), i=1,n)
	     WRITE(NT,907)(AMIN(I),TEK3(I),AMAX(I),I=1,Nban)
	            end if
c
          If(debug.ge.3.) then
               CALL OUTLER(T,N,MP,1,ITOCON,B,TEK,SIM,NT)
	  END IF
c	
             CALL MISR1(T,R2,R,WLUM,N,ITOCON,MP,1)
             KK=0
             DO 845 I=1,N*N,N+1
                        KK=KK+1
                       RV(KK)=SQRT(R(I))
                       INN=I+N*N
                       KKN=KK+N
                       RV(KKN)=SQRT(R(INN))
845                 CONTINUE
                  if (debug.ge.3.) then
             CALL PRT(R2,N,1,1,C7,TEK,MEN,NT,jmode)
             CALL PRT(RV,N,1,1,C8,TEK,MEN,NT,jmode)
                  end if
      END IF
C
C     Calculation of DETERMINANTS FOR BAYESSIAN LEARNING AND CLASSIFICATION
C
       IF(JMODE.EQ.'CLASSIFICATION'.OR.JMODE.EQ.'ONE-LEAVE-OUT-')
     *  THEN
                 CALL MISR1(A,R2,R,U,N,ITO,MB,L)
                CALL UCOPY(R,R3,N*N*L)
C
c *************  NEW DETERMINAT, DOUBLE PRECISION *********
c
	                  DO I=1,L	
			NSH=(I-1)*N*N+1
C
			deter(I)=1./sqrt(det(r3(NSH),N,N))
	if(debug.ge.4) then		
	write(nt,'(12x, i3,''-th class determinant equals: '',f12.4)')
     *                                          i,deter(I)
                  end if
                           call RSINV(N,R(nsh),N,IF)
		END DO
C
        END IF
c
C*************** WEDGE TEST (CRAB DATA)   ****************************
C
      IF (JMODE.EQ.'FAST'.and.l.eq.2) THEN
c
        CALL SELECT(T,DP,NL,SS1,F,AP,N,MP,KCL,IAP,MGAM)
      WRITE(NT,'(5x,i5,'' events from'',i7,'' passed WEDGE cut''/
     *5x,''experimental data from: '',a50)')MGAM,MP,icon
        CALL VSCALE(SS1,1./MP,FLI,IAP*KCL)
        CALL PRT(ss1,KCL,iap,1,C27,PP,FF,NT,jmode)
C
C************************** SOBOL-CUT *********************************
C
      ELSE IF (JMODE.EQ.'SOBOL-CUT'.and.l.eq.2) THEN
      call sobol(A,n,m,l,ito,cmem1,cmem2,cmin,cmax,amax,amin,
     *search,genert,tek,nt)
C************************** SUPER-CUT *********************************
c
      ELSE IF (JMODE.EQ.'SUPERCUT'.and.l.eq.2) THEN
	write(nt,'(5x,''ATTENSION !!!, IT WAS SUPERCUT !!!!'')')	
	siold=(itot(1)-itot(2))/sqrt(real(itot(1))+real(itot(2)))
	sinew=(ito(1)-ito(2))/sqrt(real(ito(1))+real(ito(2)))
	siold1=(itot(1)-itot(2))/real(itot(2))
	sinew1=(ito(1)-ito(2))/real(ito(2))
       WRITE(*,'(2x,''OLD N of events and SIGMAs '',2i7,2f9.5)')
     *itot(1),itot(2),siold,siold1
       WRITE(*,'(2x,''NEW N of events and SIGMAs '',2i7,2f9.5)')
     *ito(1),ito(2),sinew,sinew1
       WRITE(nt,'(2x,''OLD N of events and SIGMAs '',2i7,2f9.5)')
     *itot(1),itot(2),siold,siold1
       WRITE(nt,'(2x,''NEW N of events and SIGMAs '',2i7,2f9.5)')
     *ito(1),ito(2),sinew,sinew1

c
C************************** MULTI-CUT *********************************
C
      ELSE IF (JMODE.EQ.'MULTI-CUT'.and.l.eq.2) THEN
       call multi(A,n,m,l,ito,cmem1,cmem2,cmin,cmax,amax,amin,
     *ss1,ss2,f,q,dfm,fli,jmode,search,genert,tek,nt)
c
C************************** DETECTOR CALIBRATION **********************
C
      ELSE IF (JMODE.EQ.'HGG') THEN
       k=6
       m=ito(1)
      call hgg(a,b,n,m,k)
       WRITE(NT,'(''n,m,k= '',3i6)')n,m,k
       i1=1
       i2=n
       do i=1,k
      WRITE(NT,'(''DETECTOR N '',i6/
     *6(10f8.0/))')i,(B(II),II=i1,i2)
       a(i)=vsum(b(i1),n)
          i1=i1+n
          i2=i2+n
       end do
c
c
                tek4(1)= 'detN13'
                tek4(2)= 'detN29'
                tek4(3)= 'detN45'
                tek4(4)= 'detN61'
                tek4(5)= 'detN76'
                tek4(6)= 'detN93'
                tek4(7)= 'CODE'
                do i=1,k
         WRITE(NT,'(''DETECTOR N '',i6,'', Background'',f9.0)')i,a(i)
                 end do
              call HBOOKn(ntidw+7,'DETECTOR CALIBRATION'
     *   , k+1,'//ani98',memw,tek4)
            i1=1
            i2=k
        do i=1,n
           do j=1,k
             tontuple(j)=b(i+(j-1)*n)
           end do
           tontuple(k+1)=i
            call HFN(ntidw+7,tontuple) 	
        end do
c
C*************** NONPARAMETRIC REGRESSION   **************************
C
      ELSE IF (JMODE.EQ.'REGRO') THEN
       WRITE(NT,'(//16X,''NUMBER OF control, NEAREST NEIGHBOURS'',2I5/
     *16x,''WEIGHTS PROPORTIONAL to DISTANCE- '',a8)')mp,NEI,JDIST
c
      NS=N-1
            M=MB
c
      CALL ASUB(A,DS,TEK,N,NS,M)
      CALL MISR1(DS,R2,R,U,NS,ITO,M,L)
cc
		DO I=1,L
        CALL UCOPY(R,R3,NS*NS*L)
			NSH=(I-1)*NS*NS+1
			deter(I)=1./sqrt(det(r3(NSH),NS,NS))
        call RSINV(NS,R(nsh),NS,IF)
		END DO
C
	call UZERO(M22,1,MB)
	call uzero(NL,1,MP*NEI)
      CALL DISTNN(A,T,UA ,DI,M22,NL,DP,R,N,M,1,MP,NEI,NS,
     * JDIFF)
c
      CALL KNNES(A,T,U,WLUM,NL,DP,DEL,D,WP,EST,SS,N,M,MP,NEI,
     * DIS,DEN,NT,DEBUG,JDIST)
c		
               do i=1, mp
                   icc=i*n
                   ds(i)=t(icc)
                 end do
c
           if( ntupw.ge.1) then
                tek4(1)='relative'
                tek4(2)='Eotrue'
                tek4(3)='Eoestm'
              call HBOOKn(ntidw*100+7,'ENERGY ESTIMATION BY REGRO'
     *   , 3,'//ani98',memw,tek4)
        do i=1, mp
             tontuple(1)=del(i)
             tontuple(2)=ds(i)
             tontuple(3)=est(i)
            call HFN(ntidw*100+7,tontuple) 	
        end do
c
       end if
c
c      ********ATTENSION KOLM2 IS SORTING INPUT ARRAYS **********
c      KOLMOGOROV-SMIRNOF TEST FOR ESTIMATED and "TRUE" ENERGY DISTRIBUTIONS
c
		CALL KOLM2(DS,EST,MP,MP,ZK,PROB)
		WRITE(NT,'(2X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''
     *,5x,2e12.4)') ZK,PROB
c
C ********ADAPTIVE NONPARAMETRIC REGRESSION   ***********
C
      ELSE IF (JMODE.EQ.'REGRO-AD') THEN
c
      WRITE(NT,'(//16X,''NONPARAMETRIC REGRESSION FOR VARIABLE '',A8/
     *16x,''NUMBERS OF TRAINING AND CONTROL '',2I7)')
     * numreg(1),mb,mp
                       inout=numreg(1)
                       call varout(a,ds,a,n,mb,tek,tek3,inout,nt)
                       call varout(t,ua,t,n,mp,tek,tek3,inout,nt)
                       n=n-1
c

      CALL MISR1(a,R2,R,U,N,ITO,MB,L)
      IF(DEBUG.GE.2) THEN
             CALL PRT(R2,N,L,1,C7,TEK3,MEN,NT,jmode)
c             CALL PRT(R,N,N,L,C8,TEK3,MEN,NT,jmode)
      END IF
c
           if( ntupw.ge.1) then
                tek4(1)='relative'
                tek4(3)='Eotrue'
                tek4(2)='Eoestm'
              call HBOOKn(ntidw*100+7,'ENERGY ESTIMATION BY REGRO-AD'
     *   , 3,'//ani98',memw,tek4)
        do i=1, mp
             tontuple(1)=ua(i)-dp(i+kcl*mp)
             tontuple(3)=ua(i)
             tontuple(2)=dp(i+kcl*mp)
            call HFN(ntidw*100+7,tontuple) 	
        end do
c
       end if
c
c
		DO I=1,L
        CALL UCOPY(R,R3,N*N*L)
	    NSH=(I-1)*N*N+1
	    deter(I)=sqrt(det(r3(NSH),N,N))
        call RSINV(N,R(nsh),N,IF)
		END DO	
c
       CALL REGR(A,T,U,DIS,DP,R,DI,N,ITO,MB,L,MP,KCL
     * ,UA,D,F,DETER,DS,B,DEBUG,NT,Y,
     *  expmax,strang,jmode)
c
        CALL PRT(DIS,1,KCL+2,1,C12,TEL,FF,NT,jmode)
        CALL PRT(D,1,KCL+2,1,C13,TEL,FF,NT,jmode)
c
        if (jmode.eq.'REGRO-AD') then
        ii=-mp+1
          do i=1,kcl+2
        ii=ii+mp
c
       CALL KOLM2(dp(ii),ua,MP,MP,ZK,PROB)
      WRITE(NT,'(/22X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''/
     *5x,2e12.4,'' Smoothing parameter-'', a8)') ZK,PROB,ff(i)
          end do
         end if
c
C ********** BOOTSTRAP REPLICAS *****************************************
C
      ELSE IF (JMODE.EQ.'BUTSTRAP') THEN
         IAP=1
         call vzero(S1RdfM,L*L*(KCL+2))
         call vzero(S2RdfM,L*L*(KCL+2))
         call vzero(S1dfM,L*(KCL+2))
         call vzero(S2dfM,L*(KCL+2))
         call vzero(pfm,L*(KCL+2))
         call vzero(pfd,L*(KCL+2))
C
	DO  III= 1,NBUT
	  CALL BUT(A,B,U,UA,N,MB,L,ITO,III,M22)
          CALL TURN(UA,E1M,ITO,MB,L)
c
	  CALL MISR1(B,R2,R,UA,N,ITO,MB,L)
	  CALL UCOPY(R,R3,N*N*L)
		DO I=1,L
			NSH=(I-1)*N*N+1
			deter(I)=1./sqrt(det(r3(NSH),N,N))
                        call RSINV(N,R(nsh),N,IF)
         if(debug.ge.4) then		
	write(nt,'(2i3,f12.4)')iii,i,deter(I)
         end if
		END DO
	  IF(N.EQ.1.AND.L.EQ.2.and.debug.ge.4.) THEN
	    call ucopy(a,at,mb)
	    call ucopy(b,ds,mb)
c
	   CALL KOLM2(AT,DS,MB,MB,ZK,PROB)
           WRITE(NT,'(2X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''
     *,5x,2e12.4)') ZK,PROB
          END IF
C
        jmode='ONE-LEAVE-OUT-'
        CALL PARZ(B,B,UA,PST,DEN,R,DI,N,ITO,MB,L,MB,KCL,m22,del,nei,
     * WLUM,AP,IAP,F,DETER,DS,RDFM,jmode
     * ,DEBUG,JDEN,NT,expmax,strang)
C       CALL PRT(RDFM,L,KCL+2,L,C4,TEL,FF,NT,jmode)
C
       jmode='CLASSIFICATION'
        CALL PARZ(B,T,UA,PST,DEN,R,DI,N,ITO,MB,L,MP,KCL,m22,del,nei,
     *  WLUM,AP,IAP,F,DETER,DS,DFM,jmode
     * ,DEBUG,JDEN,NT,expmax,strang)
C
C
	CALL PRFER(RDFM,DFM,R3,R2,PF,C0,CC,RR,DR3,DC0,L,KCL+2,
     *  nt,debug,IRR)
c     	CALL PRT(PF,L,KCL+2,1,C11,TEL,FF,NT,jmode)
 	 call fill(RDFM,S1RdfM,S2RDFM,L*L*(KCL+2),iii,nbut)
	 call fill(DFM,S1DFM,S2DFM,L*(KCL+2),iii,nbut)
	 CALL FILL(PF,PFM,PFD,L*(KCL+2),III,NBUT)
C
       END DO
               CALL PRT(S1RDFM,L,KCL+2,L,C14,TEL,FF,NT,jmode)
               CALL PRT(S1DFM,L,KCL+2,1,C22,TEL,FF,NT,jmode)
         CALL PRT(S2RDFM,L,KCL+2,L,c16,TEL,FF,NT,jmode)
         CALL PRT(S2DFM,L,KCL+2,1,c17,TEL,FF,NT,jmode)
         CALL PRT(PFM,L,KCL+2,1,C25,TEL,FF,NT,jmode)
	       CALL PRT(PFD,L,KCL+2,1,C26,TEL,FF,NT,jmode)
c
	CALL PRFER(S1RDFM,S1DFM,R3,R2,PF,C0,CC,RR,DR3,DC0,L,KCL+2,
     *		nt,debug,IRR)
C	
		CALL PRT(PF,L,KCL+2,1,C11,TEL,FF,NT,jmode)
C
C****************  DUMP SAMPLE WRITING ( SEQUENTAL FILE ) **************
C
         ELSE IF (JMODE.EQ.'FWRITE') THEN
C
        CALL FWRITE(A,U,IDUM,TSNAME(1),mb,N,TEK,FORMAT1,IFORMW,nt)
       WRITE(NT,807)TSNAME(1),ITO(1),FORMAT1,IDUM
C
C
C**************  INTRINSIC DIMENSIONALITY   ********* ******
C
      ELSE IF (JMODE.EQ.'DIMFLAT') THEN
C
            jj=-ito(1)*n+1
           do i=1,l
               jj=jj+ito(i)*n
c       CALL TIDIM(a(jj),T,WLUM,PF,PFER,PFERM,G,GOB,M22,N,
c      *maxter,ito(i),nei,
c     * NT, probir,ndel,di,tmin,genert,ntupw,ntidw,memw,tek,hbname)
           call cjoin(a,b,t,n,ito(i),i,l)
         end do
         if(ntupw.ge.1) then
             tek(4)='Locdim'
            mm=-n
        	 call HBOOKN(ntidw*111+i,'LOCAL DIMENSIONALITY'
     *   ,N+1,'//ani98',MEMW,tek)
          do i=1,L
             do ii=1,ito(i)
                mm=mm+(n+1)
                call ucopy(t(mm),ua,n)
                ua(na+1)=pf(nei)
                call hfn(ntidw*111+i,ua)
             end do
             end do
          end if
c
c **************************************************************************
c
c **************** MP>NDEL !!!!! *******************************************
c
c
           ELSE IF (JMODE.EQ.'DIMDIM') THEN
c
              if(genert.eq.'lp-tau') then
		      maxter=ndel
              else if(genert.eq.'samept') then
	               maxter=mb	
              end if
c
        CALL TIDIM(a,T,WLUM,PF,PFER,PFERM,G,GOB,M22,N,MAXTER,MB,nei,
     *  NT,probir,ndel,adif,tmin,genert,ntupw,ntidw,memw,tek,hbname)
c
C*****************  COVARIATION ANALYSIS    ********************************
C
       ELSE IF (JMODE.EQ.'COVCOR') THEN
C
        CALL MISR1(A,R2,R,U,N,ITO,MB,L)
        KK=0
	do j=1,l
        	DO 245 I=1,N*N,N+1
        		KK=KK+1
        		INN=I+N*N*(j-1)
       			 RV(KK)=SQRT(R(INN))
245   	        CONTINUE
	end do
        CALL PRT(R2,N,L,1,C7,TEK,MEN,NT,jmode)
        CALL PRT(RV,N,L,1,C8,TEK,MEN,NT,jmode)
        CALL COVCOR(R,R3,E2,IP,N,MB,L,NT,TEK,JMODE)
C
C*******************NEURAL NET LEARNING***************************************
C
ccccccccccccccccccccccccccccccccccccccccccccc
	ELSE if(JMODE.EQ.'JETNET') THEN
		if(debug.ge.4) then
		call jnread(50)
		else
	mstjn(1)=leyers
	mstjn(2)=mb
	mstjn(5)=15
	mstjn(9)=1
	do i=1,leyers
	mstjn(9+i)=nodes(i)
	end do
	parjn(1)=cf
	parjn(30)=speed
	parjn(31)=wspeed
		end if
	parjn(32)=50.0
	parjn(33)=1.E-6
	call JNINIT
	do 10 i=1,Niter
	   kk=0
	   no=0
	do 20 kl=1,L
	do 30 j=1,ito(kl)
	   no=no+1
	   do k=1,mstjn(10)
	   kk=kk+1
	OINJN(k)=A(k+kk)
	   end do
	OUTJN(1)=(kl-1)/(L-1)
	CALL JNTRAL
	IF(I.eq.Niter-1) THEN
	omem1(no+1)=outjn(1)
	END IF
30	continue
20	continue
	write(*,*)i,parjn(9)
10	continue
          		lines=l
                        do J=1,nipr
                           k=(j-1)*l+1
        call rate(omem1,part(k),ccl,ito,L,MB,tel,npp(k),jmode,nt,lines)
                        end do
	call JNDUMP(50)	
CCC	
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       if (ntupw.eq.1.and.debug.ge.2.and.qualit.eq.'montec') then
               tek(n+1)='netoutpt'
                 call HBOOKN(ntidw*200+1,'NEURAL CLASSIFICATION',n+1,
     *            '//ani98',memw,tek)
                         jj=-n
                         do j=1,mb
                              jj=jj+n
                              do i=1,n
                                   tontuple(i)=a(jj+i)
                              end do
c
                              tontuple(n+1)=omem1(j)
                              call  hfn(ntidw*200+1,tontuple)
                         end do
c
         WRITE(*,'(5X,''TS AND NN OUTPUT NTUPLE  FILLED'')')
       end if
c
CCC      IF(istg.gt.1) Then END IF
C
      IF(DEBUG.GE.2.AND.QUALIT.NE.'estima'.AND.NTUPW.EQ.1) THEN
c
c	VISUALISATION OF THE NONLINEAR DECISION SURFACE
c	GENERATION OF PROBES AND CALCULATION OF CORRESPONDING
C 	OUTPUT OF NETWORK WITH PREVIOUSLY OPTIMAZED COUPLINGS
c
c         call vizbon(A,N,MB,L,ITO,B,TEK,SIM,NT,tmin,tmax,adif)
c
c
                   tek(n+1)='NNoutput'		
                   call HBOOKN(ntidw*10,'NEURAL CLUSTER',N+1,
     *            '//ani98',memw,tek)
                    do itau=1,ndel
            call scanpt(probir,n,tmin,tmax,adif,genert,itau)
c
	  call ucopy(probir,oinjn,n)	
c
	  call jntest	
			       	    probir(n+1)=outjn(1) 					
        		           call HFN(ntidw*10,probir) 						
                    end do
        WRITE(*,'(5X,''NEURAL CLUSTER SCANNING NTUPLE  FILLED'')')
        END IF
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC	
CCCCCCCCCCCCCCCC START THE CROSS VALIDATION CCCCCCCCCCCCCCCCCCCCCC	
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

        ELSE IF(JMODE.EQ.'CVM') THEN
C
      WRITE (NT,'(5X,''N OF RANDOM WALK ITERATIONS, RANDOM STEP'',
     *I8,2X,F7.3)') NITER,CF
      WRITE (NT,'(5X,''ADAPTATION SPEED AND WIEGHT SPEED'',2F7.4)')
     *Speed,Wspeed
      WRITE (NT,'(5X,''INITIAL NET SPREAD '',F7.4)')SPREED
c
       WRITE (NT,447) search,qualit,qtype,memory,normal,begin,genert
     *  ,stiter,dpoint
     *  ,(NODES(I),I=1,LEYERS)
447      FORMAT( 5X,'RANDOM SEARCH MODE', A10/
     *5X,'QUALITY FUNCTION TYPE:',A10/
     *5X,'QUALITY  TYPE:',A10/
     *5X,'MEMORY TYPE:',A10/
     *5X,'DATA NORMALISATION (TO 0 -1 INTERVAL):',A10/
     *5X,'STARTED FROM THE  POINT:',A10/
     *5X,'RANDOM GENERATOR USED :',A10/
     *5X,'STOP ITERATIONS, IF Q <',E12.4/
     *5X,'DECISION POINT:',F12.4/
     *5X,'NEURAL NET CONFIGURATION: ',' :: ',6I3 )
c
       WRITE(NT,'(5X,''TOTAL EVENTS, SHIFT OF RANDOM GENERATOR'',2I6)')
     *  MB,ISHIFT
C
          if(qualit.eq.'estima') then
           WRITE(NT,'(5X,''VECTOR OUTPUT FUNCTION WEIGHTS'',9f7.3)')
     *   (WIGHT(I),I=1,NY)
          else
           WRITE(NT,'(5X,''DIFFERENT TS CLASSES WEIGHTS'',5f7.3)')
     *   (WIGHT(I),I=1,L)
         end if
        if(search.eq.'mullti'.and.memory.eq.'simple'.and.genert.eq.
     *'lp-tau') then
       WRITE(NT,'(/5x,''DETERMINISTIC OPTIMIZATION  ON SOBOL SEED''/
     *15x,''WITH THE  SCALE OF'', f7.3)')cf
       end if
c
          if(qualit.eq.'estima') then
          WRITE(nt,'(5X,''NEURAL REGRESSION, REGRESSAND(S) '',2A10)')
     * (numreg(i),i=1,ny)
c
               IF(NODES(1).NE.N-NY.or.nodes(leyers).ne.ny)
     *    THEN
        WRITE(isys,'(5X,'' NET and VARIABLES MISMATCH: '',2i3/
     *  5x,''NET TOPOLOGY: '',10I3)') N,NY,(NODES(i),i=1,leyers)
        STOP 97
               END IF
          ELSE
               IF(NODES(1).NE.N)
     *    THEN
        WRITE(isys,'(5X,'' NET and VARIABLES MISMATCH: '',i3/
     *  5x,''NET TOPOLOGY: '',10I3)') N,(NODES(i),i=1,leyers)
        STOP 97
               END IF
          end if
C	
C
CCC CREATING THE NEW TRAINIG AND CONTROL SAMPLES CCC
			 call vzero(cmem1,in)
			 call vzero(cmem2,in)
			 call vzero(cclnew,nipr)
			 call vzero(cc1,in)
			 call vzero(cc2,in)
			 call vzero(cclnu,nipr)
			 call vzero(fpe,nipr)
	icsh=-ictot
	mmcv=int(mb/(l*ictot))
	nitcv=mmcv
	DO 1234 icvs=1,nitcv
	icsh=icsh+ictot
	   do i=1,l
	      nito(i)=0
	      itocon(i)=0
	   end do
	nictot=0
	nmb=0
	nmp=0
		cvt=char(48+int(icvs/10))//char(48+mod(icvs,10))
		hbname='learning'//cvt//'.hbook'
	    if(ntupw.gt.0) then
	       call HROPEN(ntidw,'ani98',hbname,'N',lreclw,ISTAT)
	    end if

	call CVSCR(a,t,cv,ito,ictot,l,mb,icsh,n)
	
	   do i=1,l
	      nito(i)=ito(i)-ictot
	      itocon(i)=ictot
	   end do
	nictot=L*ictot
	nmb=mb-L*ictot
	nmp=L*ictot
	  	                      INDTOT=0
                      DO I=1,LEYERS-1
                             INDTOT=INDTOT+NODES(I)*NODES(I+1)
                     END DO
                      NNODES=0
                      DO  I=2,LEYERS
	            NNODES=NNODES+NODES(I)
	    END DO
                      ITPAR=INDTOT+NNODES
			
          if(begin.eq.'random')then
	            DO  I=1,ITPAR
                        TJ(I)= RG32(DUMMY)*SPREED
                    END DO
	  end if	
C	
c
       if(qualit.eq.'estima') then
c
c        NORMALIZATION OF DATA FOR NEURAL ESTIMATION
C
	if(icvs.eq.1) then
                    ii=-nmb+1
                    do i=1,ny
                         call numnam(tek,numreg(i),n,jreg,isys)
                       di(i)=tmax(jreg)-tmin(jreg)
                       ii=ii+nmb
                       inout=numreg(i)
                       call varout(cv,b(ii),cv,n,nmb,tek,tek3,inout,nt)
                       n=n-1
	       if(normal.eq.'renorm') then
	   call denorm(b(ii),b(ii),tmin(jreg),tmax(jreg),1,nmb)
C
           write(nt,'(/5x,''TS REGRESSANDS ARE NORMALIZED TO 0-1, ACTUAL
     *     VALUES IN:''/50(2X,E12.4,'' < '',A8,'' <'',E12.4/))')
     *    (tmin(j),tek(j),tmax(j),j=1,ny)
	       end if	
                    end do
	end if
          end if
c
          istgm=nodes(1)*nodes(2)+2
          nodout=nodes(leyers)
          maxcon=itpar
C
C
         CALL RLEARN(cv,B,U,O1,OMEM1,Q,O,SSS,FMERIT,TJ,PAR,NUMREG,
     *TJMEM,tjmemp,TJPAW,NODOUT,MAXCON,NNODES,ISTG,MEMW,TAGS,NTUPW,
     *NODES,TEK,NITO,L,N,NY,NMB,INDTOT,ITPAR,LEYERS,NTIDW,NT,ISYS,ISTGM,
     *DEBUG,EXPMAX,SPEED,WSPEED,WIGHT,STITER,NITER,CF,DPOINT,NTJ,
     *QUALIT,QTYPE,SEARCH,MEMORY,GENERT,NORMAL,SIM,goal,ISHIFT,re,
     *part,npp,tel,ccl,nipr,cclnew,t,tb,itocon,nmp,jmode,spreed,cclr)
C
        if(qualit.eq.'estima') then	
          if(ntupw.eq.1) then
               do i=1,ny
                    tek4(i)=numreg(i)
                    tek4(ny+i)= tek4(i)
c                    	
c   ADDING TO THE  NAME OF VARIABLE "estm" SUFFIX FOR A NEW TS WITH "ENERGY"
c
                 lennam=index(tek4(ny+i),' ')
                 if(lennam.le.4) then
                     tek4(ny+i)(lennam:lennam+3)='estm'
                 else
                    tek4(ny+i)(5:8)='estm'
                 end if
c
               end do
               do i=1,n
               tek4(2*ny+i)=tek3(i)
               end do
                 call HBOOKN(ntidw*100 +1,'NEURAL REGRESSION',ny*2+n,
     *            '//ani98',memw,tek4)
                     jj=-n
                    do j=1,nmb
                        ii=-nmb
                        jj=jj+n
                        do i=1,n
                              tontuple(2*ny+i)=a(jj+i)
                         end do
          do i=1,ny
               ii=ii+nmb
               if(normal.eq.'renorm') then
                 call numnam(tek,numreg(i),n,jreg,isys)
	     tontuple(i)=b(j+ii)*di(i)+tmin(jreg)
                       tontuple(ny+i)=omem1(j+ii)*di(i)+tmin(jreg)
               else
                       tontuple(i)=b(j+ii)
                       tontuple(ny+i)=omem1(j+ii)
               end if
          end do
c
                            call  hfn(ntidw*100+1,tontuple)
                    end do
            end if                        	
        end if
c
c 	CALCULATION OF THE CLASSIFICATION RATES
c
          IF(debug.ge.1.and.qualit.eq.'montec')  THEN

          lines=l
                         do J=1,nipr
                              k=(j-1)*l+1
        call rate(omem1,part(k),ccl,nito,L,nMB,tel,npp(k),jmode,nt,
     *            lines)
C
C*** Calculation of mean rates ***
			 do iil=1,l
			 do jjl=1,l
			 if(jjl.eq.iil) then
			 ccl(jjl,iil)=0.
			 end if
			 cclnew(j)=cclnew(j)+ccl(jjl,iil)
			 end do
			 end do
			 cclnew(j)=cclnew(j)/l			
		call fill(cclnew(j),cmem1(j),cmem2(j),nipr,icvs,nitcv)
C*** End of mean rates ***
C
			 end do
          end if
c
        if(istg.gt.1) then
C
c ************************* FILLING HISTOGRAM (ntuple) ***************
c
       if (ntupw.eq.1.and.debug.ge.2.and.qualit.eq.'montec') then
               tek(n+1)='netoutpt'
                 call HBOOKN(ntidw*200+1,'NEURAL CLASSIFICATION',n+1,
     *            '//ani98',memw,tek)
                         jj=-n
                         do j=1,nmb
                              jj=jj+n
                              do i=1,n
                                   tontuple(i)=cv(jj+i)
                              end do
c
                              tontuple(n+1)=omem1(j)
                              call  hfn(ntidw*200+1,tontuple)
                         end do
c
         WRITE(*,'(5X,''TS AND NN OUTPUT NTUPLE  FILLED'')')
       end if
c
        END IF
C
      IF(DEBUG.GE.2.AND.QUALIT.NE.'estima'.AND.NTUPW.EQ.1) THEN
c
c	VISUALISATION OF THE NONLINEAR DECISION SURFACE
c	GENERATION OF PROBES AND CALCULATION OF CORRESPONDING
C 	OUTPUT OF NETWORK WITH PREVIOUSLY OPTIMAZED COUPLINGS
c
c         call vizbon(cv,N,nMB,L,NITO,B,TEK,SIM,NT,tmin,tmax,adif)
c
c
                   tek(n+1)='NNoutput'		
                   call HBOOKN(ntidw*10,'NEURAL CLUSTER',N+1,
     *            '//ani98',memw,tek)
                    do itau=1,ndel
            call scanpt(probir,n,tmin,tmax,adif,genert,itau)
c
	  call ucopy(probir,q,n)	
          call neural(q,o,tjmem,nodes,leyers,indtot,expmax,debug)
c	
			       	    probir(n+1)=o(1) 					
        		           call HFN(ntidw*10,probir) 			
c			
                    end do
        WRITE(*,'(5X,''NEURAL CLUSTER SCANNING NTUPLE  FILLED'')')
        END IF
        IF(DEBUG.GE.2.AND.QUALIT.EQ.'estima') THEN
                    ii=-nmb+1
                         do i=1,ny
                                    ii=ii+nmb                     		
		CALL KOLM2(b(ii),omem1(ii),nmb,nmb,ZK,PROB)
	WRITE(NT,'(//5X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''
     *,2e12.4)') ZK,PROB
                         end do
          END IF
	  if(ntupw.gt.0) then
	  call hrout(0,icycle,' ')
	  call hrend('ani98')
	  end if
c
c******************** CALCULATION OF MEAN ERRORS **********************
C
           if(qualit.eq.'estima') then	
        call errors(oMEM1,b,rbha,rmah,nmb,ny,nt)
	call fill(rmah,cmem1,cmem2,nipr,icvs,nitcv)
	   end if
c
C *****************NEURAL NET CLASSIFICATION***************************
C
C         ELSE IF(JMODE.EQ.'CLASSIFI'.or.jmode.eq.'EXP') THEN
c
c      changed at 30 07 1997, why =1!?
c
          REWIND NTJ
C	
		hbname='classifi'//cvt//'.hbook'
	    if(ntupw.gt.0) then
	       call HROPEN(ntidw,'ani98',hbname,'N',lreclw,ISTAT)
	    end if	
C
       READ (NTJ,'(4I6/10(10I6/))') LEYERS,INDTOT,NNODES,ISHIFT,
     * (NODES(I),I=1,LEYERS)
	READ(NTJ,6) IT
	READ(NTJ,*)(golOLD(I),I=1,L)
	READ(NTJ,6) IT
	READ(NTJ,16)NNORMA
      READ (NTJ,6) IT
      READ(NTJ,*) NITER,CF,SPEED,WSPEED
      READ (NTJ,6) IT
      READ (NTJ,*) (WIGHT(I),I=1,L)
            ITPAR=INDTOT+NNODES
      READ (NTJ,6) IT
      READ(NTJ,'(A6)')QUALIT
C
       READ (NTJ,6) IT
      READ(NTJ,'(A6)')QTYPE
      READ (NTJ,6) IT
      READ(NTJ,'(I5)')NY
      READ (NTJ,6) IT
      READ(NTJ,'(10A10)')(TEK4(I),I=1,NY)
      READ (NTJ,*) (TJ(I),I=1,ITPAR)
C
	if(nnorma.ne.normal) then
       write(isys,'(5x,''Normalisation disagreement !''/2a6)')
     * nnorma,normal
		stop 543
	end if
c
       WRITE (NT,'(5X,''CLASS. (ESTIMATION)  WITH NEURAL NETWORK''/
     *   5x''TRAINED AS FOLLOWING:'')')
       WRITE (NT,'(5X,''N OF RANDOM WALK ITERATIONS, RANDOM STEP'',
     *I8,2X,F7.3)') NITER,CF
c
      WRITE (NT,'(5x,''INITIAL NET SPREAD '',F7.4)')SPREED
      WRITE (NT,'(5X,''ADAPTATION SPEED AND WIEGHT SPEED:'',2F7.4)')
     *SPEED,WSPEED
c
       WRITE (NT,47) search,qualit,qtype,memory,normal,begin,genert
     *  ,stiter,dpoint
     *  ,(NODES(I),I=1,LEYERS)
c
      WRITE(NT,'(5X,''RANDOM SHIFT, ASYMMETRY '',I6,5F15.4)')
     *  ISHIFT, (WIGHT(I),I=1,L)
c
      WRITE(NT,'(5X,''REGRESSAND (S)  '',5a10)')
     * (tek4(i),I=1,ny)
c
               IF(DEBUG.GE.5.) THEN
       WRITE(NT,'(//5X,''IMPLEMENTING THE NETWORK WITH PARAMETERS:'')')
       WRITE(NT,*)(TJ(I),I=1,Itpar)
               END IF
C
      IF(NODES(1).NE.N.and.jmode.eq.'EXP') THEN
      WRITE(ISYS,'(5X,'' NET AND VARIABLES MISMATCH'',6I5)')NODES(1),N
                  STOP 97
      END IF
                if(qualit.eq.'estima') then
                    do i=1,ny
                         IF (NUMREG(I).NE.TEK4(I)) THEN
        WRITE(ISYS,'(5X,'' WRONG REGRESSAND'',2A10)')
     *     NUMREG(I),TEK4(I)
                              STOP 77
                        END IF
                     end do
                end if
               CALL DATIMH(NDATA,NTIME)
               CALL TIMED(TIME)
c        WRITE(NT,'(/2X,''BEFORE CALL NEURAL '',A8,2X,A8)')NDATA,NTIME
        WRITE(*,'(/2X,''BEFORE CALL NEURAL '',A8,2X,A8)')
     * NDATA,NTIME
c
c               IF(qualit.eq.'estima') then
c
c                              IF(NODES(1).NE.N-NY) THEN
c          WRITE(ISYS,'(5X,''NET AND VARIABLES MISMATCH'',6I5)')
c     *   NODES(1),N-NY
c                    STOP 57
c                              END IF
c                    ii=-nmp+1
c                    do i=1,ny
c                         call numnam(tek,numreg(i),n,jreg,isys)
c                         cdif(i)=cmax(jreg)-cmin(jreg)
c                         ii=ii+nmp
c                         inout=tek4(i)
c                         call varout(t,b(ii),t,n,nmp,tek,tek,inout,nt)
c                         n=n-1
c                              if(normal.eq.'renorm') then
c                                call numnam(tek,numreg(i),n,jreg,isys)
c        call denorm(b(ii),b(ii),cmin(jreg),cmax(jreg),1,nmp)	
c
c      write(nt,'(/5x,''CS REGRESSANDS ARE NORMALIZED TO 0-1, ACTUAL
c     * VALUES IN:''/50(2X,E12.4,'' < '',A8,'' <'',E12.4/))')
c     * (cmin(j),tek4(j),cmax(j),j=1,n)
c                              end if
c                    end do
c               END IF
C
          DO 4145 I=1,NMP
                              INDD=(I-1)*N+1
                              CALL UCOPY(t(INDD),Q,N)
c
          if(normal.eq.'renorm'.or.qualit.ne.'estima') then	
          CALL NEURAL(Q,O,TJ,NODES,LEYERS,INDTOT,expmax,debug)
                                    o1(i)=o(1)
c
                                   else
          CALL NEURAL2(Q,O,TJ,NODES,LEYERS,INDTOT,expmax,debug)
c
          iik=-nmp
          DO ii=1,ny
                                   iik=iik+nmp
                                   o1(I+IIK)=O(ii)
          END DO
                                   end if
C
c         if(t(2).lt.1)then
c               write(isys,'(2i8,7f9.1)')i,nmp,t(2)
c               stop 777
c           end if
4145      CONTINUE
c
c           if(i.lt.10)then
c              do indd=1,10*n,n
c                write(isys,'(7f9.1)')(t(ii),ii=indd,indd+n-1)
c              end do
c           end if
c
c******************** CALCULATION OF MEAN ERRORS *****************************
C
          if(qualit.eq.'estima') then	
                call errors(o1,b,ccnu1,ccnu2,nmp,ny,nt)
	   call fill(ccnu2,cc1,cc2,nipr,icvs,nitcv)
	  end if
C
C
               CALL TIMED(TIME)
               CALL DATIMH(NDATA,NTIME)
c               WRITE(NT,'(/2X,''NEURAL FINISHS AT '',A8,2X,A8,
c     *''       CPU TIME'',E18.8)') NDATA,NTIME,TIME
               WRITE(*,'(/2X,''NEURAL FINISHS AT '',A8,2X,A8,
     *''       CPU TIME'',E18.8)') NDATA,NTIME,TIME
c

          IF(debug.ge.2.and.qualit.ne.'estima')  THEN
c
c	VISUALISATION OF THE EXPERIMENTAL EVENTS
c	CLASSIFICATION BY TRAINED NN
c
                   tek(n+1)='NNoutput'		
                   tek(n+2)='Number'
                   call HBOOKN(ntidw*13,'EXPERIMENTAL EVENTS',N+2,
     *            '//ani98',memw,tek)
c
                    jj=-n
                    do j=1,nmp
                              jj=jj+n
c
                              do i=1,n
                                   tontuple(i)=t(jj+i)
                              end do
                              tontuple(n+1)=o1(j)
                              tontuple(n+2)=real(j)
c           if(tontuple(2).le.1.)then
c              write(nt,'(a8,7f9.1)')tek(2),(tontuple(ii),ii=1,n+2)
c           end if				
                              call HFN(ntidw*13,tontuple) 			
		    end do

        WRITE(*,'(5X,''EXPERIMENTAL EVENTS NTUPLE  FILLED'')')
c
c
c 	CALCULATION OF THE CLASSIFICATION RATES
c
          lines=l
                         do J=1,nipr
                              k=(j-1)*l+1
        call rate(o1,part(k),ccl,itocon,L,nMP,tel,npp(k),jmode,nt,lines)
C
C*** Calculation of mean rates ***
			 do iil=1,l
			 do jjl=1,l
			 if(jjl.eq.iil) then
			 ccl(jjl,iil)=0.
			 end if
			 cclnu(j)=cclnu(j)+ccl(jjl,iil)
			 end do
			 end do
			 cclnu(j)=cclnu(j)/l			
		call fill(cclnu(j),cc1(j),cc2(j),nipr,icvs,nitcv)
C*** End of mean rates ***
C
                         end do
c
         END IF
c
          IF(debug.ge.6.and.qualit.eq.'estima')  THEN
                               ii=-nmp+1
                              do i=1,ny
                                ii=ii+nmp
        CALL KOLM2(b(ii),o1(ii),nmp,nmp,ZK,PROB)
	WRITE(NT,'(2X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''
     *,2F15.10)') ZK,PROB
                              end do
          END IF
C
          if(qualit.eq.'estima')then
c
               do i=1,ny
                    tek3(n+ny+i)= tek4(i)
                    tek3(n+i)=tek4(i)
                    lennam=index(tek3(n+i),' ')
c
                    if(lennam.le.4) then
                         tek3(n+i)(lennam:lennam+3)='estm'
                    else
                         tek3(n+i)(5:8)='estm'
                    end if
c
               end do
c
               do i=1,n
                         tek3(i)=tek(i)
               end do
c
c               if(jmode.eq.'CLASSIFI') then
c
                    if(ntupw.eq.1) then
                 call HBOOKN(ntidw*100+1,'NEURAL REGR. CONTROL',ny*2+n,
     *            '//ani98',memw,tek3)
                         jj=-n
c
                       do j=1,nmp
                              ii=-nmp+1
                              jj=jj+n
c
                              do i=1,n
                                   tontuple(i)=t(jj+i)
                              end do
cc
                              do i=1,ny
                                   ii=ii+nmp
c
                                   if(normal.eq.'renorm') then
                         call numnam(tek,numreg(i),n,jreg,isys)
                         tontuple(n+ny+i)=b(j+ii)*cdif(i)+cmin(jreg)        		
                         tontuple(n+i)=o1(j+ii)*cdif(i)+cmin(jreg)
                                   else
                         tontuple(n+ny+i)=b(j+ii)
                         tontuple(n+i)=o1(j+ii)
                                   end if
                              end do
                         call  hfn(ntidw*100+1,tontuple)
                       end do
c
               end if
c
                         if (debug.ge.4.) then
c
            CALL VARIN(t,O1,cv,N,nMP,TEK,TEK,TEK3(N+1),NT,ISYS)
c
                              if(NY.EQ.2) then
            CALL VARIN(cv,O1(nmp+1),t,N+1,nMP,TEK,TEK,TEK3(N+2),NT,ISYS)
            CALL UCOPY(t,cv,(n+2)*nmp)
                              end if
c
            CALL VARIN(cv,b,t,N+NY,nMP,TEK,TEK,TEK4(1),NT,ISYS)
            CALL UCOPY(t,cv,(n+ny+1)*nmp)
c
                              if(NY.EQ.2) then
            CALL VARIN(t,b(mp+1),cv,N+NY+1,nMP,TEK,TEK,TEK4(2),NT,ISYS)
                              end if
c
c   WRITING CONTROL SAMPLE + ESTIMATED ENERGY AND MASS + TRUE ENERGY AND MASS
c	
        CALL FWRITE(cv,wlum,IDUM,Icon,nmp,N+2*NY,TEK3,FORMAT1,IFORMW,nt)
                         end if
		end if
c
c ************************** BOOKING OF SELECTED AND REJECTED EVENTS ***********
c *********** PREPARATION OF NUMBERS OF EVENTS TO BE WRITE IN IDUM FILE ****
c
           if(debug.ge.2.and.qualit.ne.'estima') then
       tek(n+1)='NNoutput'
        tek(n+2)='Number'
       call HBOOKN(ntidw*10+3,'Selected events',N+2,
     *            '//ani98',memw,tek)
c
              do i=1,nmp
                        indd=(i-1)*n+1
                    if(o1(i).le.dpoint) then
                                     ievent= ievent+1
                                    if(ntupw.eq.1) then	
                     call ucopy(t(indd),tontuple,n)
                     tontuple(n+1)=o1(i)
                     tontuple(n+2)=real(i)
c
                     call HFN(ntidw*10+3,tontuple)
                                   end if
                    end if        	    	
              end do
c	
               if(debug.ge.4.and.access.eq.'SEQUENTAL') then
c	            nito(1)=nMP
c	            nito(2)=0
                    do i=1,nban
                              amin(I)=-1.e10
                              amax(I)=1.e10
                              numb(i)=i
                    end do	
c
	            mnewc=0
c	
        CALL FREAD(t,wlum,RB,Icon,NBAN,MNEWC,TEK,DEBUG,nictot,m22,m33,
     *  NBAN,NUMB,AMIN,AMAX,nIcTOT,FORMAT1,IFORM1,IcSH,NT,ACCESS,tek4)
c
         INOUT= 'NETOUTPT'
        CALL VARIN(t,O1,cv,NBAN,nMP,TEK,TEK4,INOUT,NT,ISYS)
                    lennam=index(IDUM,' ')
                    IDUM(lennam:lennam+2)='ALL'
        CALL FWRITE(cv,wlum,IDUM,Icon,nMP,Nban+1,TEK4,
     *  FORMAT1,IFORMW,nt)
c
                    WRITE(NT,807)Icon,nmp,FORMAT1,IDUM
               end if
c       end if
       end if
       rewind ntj
       if(ntupw.gt.0) then
       call hrout(0,icycle,' ')
       call hrend('ani98')
       end if
1234	Continue
C

		rxy=(1.*itpar)/(1.*nmb)
		do j=1,nipr
		fpe(j)=cc1(j)*(1.+2.*rxy)
c		write(nt,*) cc1(j),itpar,nmb,rxy,fpe(j)
		end do
		
		if(qualit.eq.'montec') then
		
	write(nt,'(''Classification mean error(s) for training sample''/
     *       ,5f9.4)') (cmem1(i),i=1,nipr)
	write(nt,'(''Deviation(s) of mean error(s) for training sample''
     *       /,5f9.4)') (cmem2(i),i=1,nipr)
	write(nt,'(''Classification mean error(s) for control sample''/
     *       ,5f9.4)') (cc1(i),i=1,nipr)
	write(nt,'(''Deviation(s) of mean error(s) for control sample''/
     *       ,5f9.4)') (cc2(i),i=1,nipr)
	write(nt,'(''Final Prediction Error(s) of this network''/
     *       ,5f9.4)') (fpe(i),i=1,nipr)

     		else if(qualit.eq.'estima') then
		
	write(nt,'(''Estimation mean rel. error for training sample''/
     *       ,5f9.4)') (cmem1(i),i=1,nipr)
	write(nt,'(''Deviation of mean rel. error for training sample''/
     *       ,5f9.4)') (cmem2(i),i=1,nipr)
	write(nt,'(''Estimation mean rel. error for control sample''/
     *       ,5f9.4)') (cc1(i),i=1,nipr)
	write(nt,'(''Deviation of mean rel. error for control sample''/
     *       ,5f9.4)') (cc2(i),i=1,nipr)
	write(nt,'(''Final Prediction Error of this network''/
     *       ,5f9.4)') (fpe(i),i=1,nipr)

     		end if
C
CCCCCCCCCCCCCCCCCCCC END OF CROSS VALIDATION CCCCCCCCCCCCCCCCCCCCCCCCCCCC
ccccccccccccccccccccccccccccccccccccccccccccc	
        ELSE IF(JMODE.EQ.'LEARNING') THEN
C
      WRITE (NT,'(5X,''N OF RANDOM WALK ITERATIONS, RANDOM STEP'',
     *I8,2X,F7.3)') NITER,CF
      WRITE (NT,'(5X,''ADAPTATION SPEED AND WIEGHT SPEED'',2F7.4)')
     *Speed,Wspeed
      WRITE (NT,'(5X,''INITIAL NET SPREAD '',F7.4)')SPREED
c
       WRITE (NT,47) search,qualit,qtype,memory,normal,begin,genert
     *  ,stiter,dpoint
     *  ,(NODES(I),I=1,LEYERS)
47      FORMAT( 5X,'RANDOM SEARCH MODE', A10/
     *5X,'QUALITY FUNCTION TYPE:',A10/
     *5X,'QUALITY  TYPE:',A10/
     *5X,'MEMORY TYPE:',A10/
     *5X,'DATA NORMALISATION (TO 0 -1 INTERVAL):',A10/
     *5X,'STARTED FROM THE  POINT:',A10/
     *5X,'RANDOM GENERATOR USED :',A10/
     *5X,'STOP ITERATIONS, IF Q <',E12.4/
     *5X,'DECISION POINT:',F12.4/
     *5X,'NEURAL NET CONFIGURATION: ',' :: ',6I3 )
c
       WRITE(NT,'(5X,''TOTAL EVENTS, SHIFT OF RANDOM GENERATOR'',2I6)')
     *  MB,ISHIFT
C
          if(qualit.eq.'estima') then
           WRITE(NT,'(5X,''VECTOR OUTPUT FUNCTION WEIGHTS'',9f7.3)')
     *   (WIGHT(I),I=1,NY)
          else
           WRITE(NT,'(5X,''DIFFERENT TS CLASSES WEIGHTS'',5f7.3)')
     *   (WIGHT(I),I=1,L)
         end if
        if(search.eq.'mullti'.and.memory.eq.'simple'.and.genert.eq.
     *'lp-tau') then
       WRITE(NT,'(/5x,''DETERMINISTIC OPTIMIZATION  ON SOBOL SEED''/
     *15x,''WITH THE  SCALE OF'', f7.3)')cf
       end if
c
          if(qualit.eq.'estima') then
          WRITE(nt,'(5X,''NEURAL REGRESSION, REGRESSAND(S) '',2A10)')
     * (numreg(i),i=1,ny)
c
               IF(NODES(1).NE.N-NY.or.nodes(leyers).ne.ny)
     *    THEN
        WRITE(isys,'(5X,'' NET and VARIABLES MISMATCH: '',2i3/
     *  5x,''NET TOPOLOGY: '',10I3)') N,NY,(NODES(i),i=1,leyers)
        STOP 97
               END IF
          ELSE
               IF(NODES(1).NE.N)
     *    THEN
        WRITE(isys,'(5X,'' NET and VARIABLES MISMATCH: '',i3/
     *  5x,''NET TOPOLOGY: '',10I3)') N,(NODES(i),i=1,leyers)
        STOP 97
               END IF
          end if
CCC NEW TRAINING AND VALIDATION SAMPLES CCCCCCCCCCCCCCCCCCCCCCCCCC

	   do i=1,l
	      nito(i)=0
	      itocon(i)=0
	   end do
c	nictot=0
	nmb=0
	nmp=0

	call CVSCR(a,t,cv,ito,ictot,l,mb,icsh,n)
	
	   do i=1,l
	      nito(i)=ito(i)-ictot
	      itocon(i)=ictot
	   end do
c	nictot=L*ictot
	nmb=mb-L*ictot
	nmp=L*ictot
cccccccccccccccccccccccccccccccccc
	call vzero(a,n*mb)
	call ucopy(cv,a,n*nmb)
	mb=nmb
	do i=1,l
	ito(i)=nito(i)
	end do
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
                      INDTOT=0
                      DO I=1,LEYERS-1
                             INDTOT=INDTOT+NODES(I)*NODES(I+1)
                     END DO
c
                      NNODES=0
                      DO  I=2,LEYERS
	            NNODES=NNODES+NODES(I)
	    END DO
C
                      ITPAR=INDTOT+NNODES
C
          if(begin.eq.'random')then
	            DO  I=1,ITPAR
                                     TJ(I)= RG32(DUMMY)*SPREED
                              END DO
c
          else if(begin.eq.'better') then
	            rewind (ntj)
C
	READ (NTJ,'(4I6/10(10I6/))') lLEYERS,iINDTOT,NNODESs,iISHIFT,
     * (NODESs(I),I=1,LEYERS)
        write (*,'(4I6/10(10I6/))') lLEYERS,iINDTOT,NNODESs,iISHIFT,
     * (NODESs(I),I=1,LEYERS)
	READ (NTJ,6) IT
	READ(NTJ,*)(golOLD(I),I=1,L)
	write(*,*)(golOLD(I),I=1,L)
                  READ(NTJ,6) IT
	READ(NTJ,16) nnorma
	write(*,16) nnorma
	READ (NTJ,6) IT
	READ(NTJ,*)   nNITER,cCF,sSPEED,wWSPEED
	write(*,*)   nNITER,cCF,sSPEED,wWSPEED
	READ (NTJ,6) IT
	READ (NTJ,*) (wWIGHT(i),i=1,l)
	write (*,*) (wWIGHT(i),i=1,l)
                  iiTPAR=iINDTOT+NNODESs
        READ (NTJ,6) IT
        READ (NTJ, '(A6)')qqualit
        write (*, '(A6)')qqualit
        READ (NTJ,6) IT
        READ (NTJ, '(A6)')qqtype
        write (*, '(A6)')qqtype
        READ (NTJ,6) IT
        READ(NTJ,'(I5)')nny
        write(*,'(I5)')nny
        READ (NTJ,6) IT
        READ(NTJ,'(5a8)')(TEK4(I),i=1,nny)
        write(*,'(5a8)')(TEK4(I),i=1,nny)
        READ (NTJ,*) (TJ(I),I=1,iitpar)
        write (*,*) (TJ(I),I=1,iitpar)
c
               if(debug.ge.3) then
       WRITE(NT,'(//5X,''STARTED FROM BEST POINT, OBTAINED WITH PARMS:
     * '')')
                  WRITE(NT,*)(TJ(I),I=1,itpar)               	
c
      WRITE (NT,'(5X,''N OF RANDOM WALK ITERATIONS, RANDOM STEP'',
     *I8,2X,F7.3)') nNITER,cCF
      WRITE (NT,'(5X,''AD. SPEED AND WIEGHT SPEED'',2F7.4)')
     * sSPEED,wWSPEED
c
       WRITE(NT,'(5X,''QUALITY FUNCTION AND TYPE:  '',2A10)')
     *qqualit,qqtype
      WRITE(NT,'(5X,''OLD AND NEW TRUE(DESIRED) NN OUTPUTS  ''/
     * 2(5f7.3/))') (golOLD(I),I=1,L), (goal(I),I=1,L)
               end if
c
                IF(iitpar.NE.itpar) THEN
      WRITE(ISYS,'(5X,'' WRONG NET, parameters:'',6I5)')iitpar,itpar
                  STOP 77
                END IF
                IF(qqualit.NE.qualit) THEN
      WRITE(ISYS,'(5X,'' WRONG QUALITY FUNC.'',2A6)')qqualit, qualit
                  STOP 77
                END IF
                IF(qqtype.NE.qtype) THEN
      WRITE(ISYS,'(5X,'' ATENSION! CHANGE THE QUALITY TYPE.'',2A8)')
     * qqtype,qtype
                END IF
c	
               if(qualit.eq.'estima') then
                    do i=1,ny
                         IF (NUMREG(I).NE.TEK4(I)) THEN
        WRITE(ISYS,'(5X,'' WRONG REGRESSAND'',2A10)')
     *     NUMREG(I),TEK4(I)
                              STOP 77
                        END IF
                    end do
               end if
       end if
       if(qualit.eq.'estima') then
c
c        NORMALIZATION OF DATA FOR NEURAL ESTIMATION
C
                    ii=-mb+1
		    jj=-nmp+1
                    do i=1,ny
                         call numnam(tek,numreg(i),n,jreg,isys)
                       di(i)=tmax(jreg)-tmin(jreg)
                       ii=ii+mb
		       jj=jj+nmp
                       inout=numreg(i)
                       call varout(a,b(ii),a,n,mb,tek,tek3,inout,nt)
		       call varout(t,tb(jj),t,n,nmp,tek,tek3,inout,nt)
                       n=n-1
	       if(normal.eq.'renorm') then
	   call denorm(b(ii),b(ii),tmin(jreg),tmax(jreg),1,mb)
	   call denorm(tb(jj),tb(jj),tmin(jreg),tmax(jreg),1,nmp)
           write(nt,'(/5x,''TS REGRESSANDS ARE NORMALIZED TO 0-1, ACTUAL
     *     VALUES IN:''/50(2X,E12.4,'' < '',A8,'' <'',E12.4/))')
     *    (tmin(j),tek(j),tmax(j),j=1,ny)
	       end if	
                    end do
          end if
c
          istgm=nodes(1)*nodes(2)+2
          nodout=nodes(leyers)
          maxcon=itpar
C
C
         CALL RLEARN(A,B,U,O1,OMEM1,Q,O,SSS,FMERIT,TJ,PAR,NUMREG,
     *TJMEM,tjmemp,TJPAW,NODOUT,MAXCON,NNODES,ISTG,MEMW,TAGS,NTUPW,
     *NODES,TEK,ITO,L,N,NY,MB,INDTOT,ITPAR,LEYERS,NTIDW,NT,ISYS,ISTGM,
     *DEBUG,EXPMAX,SPEED,WSPEED,WIGHT,STITER,NITER,CF,DPOINT,NTJ,
     *QUALIT,QTYPE,SEARCH,MEMORY,GENERT,NORMAL,SIM,goal,ISHIFT,re,
     *part,npp,tel,ccl,nipr,cclnew,t,tb,itocon,nmp,jmode,spreed,cclr)
C
        if(qualit.eq.'estima') then	
          if(ntupw.eq.1) then
               do i=1,ny
                    tek4(i)=numreg(i)
                    tek4(ny+i)= tek4(i)
c                    	
c   ADDING TO THE  NAME OF VARIABLE "estm" SUFFIX FOR A NEW TS WITH "ENERGY"
c
                 lennam=index(tek4(ny+i),' ')
                 if(lennam.le.4) then
                     tek4(ny+i)(lennam:lennam+3)='estm'
                 else
                    tek4(ny+i)(5:8)='estm'
                 end if
c
               end do
               do i=1,n
               tek4(2*ny+i)=tek3(i)
               end do
                 call HBOOKN(ntidw*100 +1,'NEURAL REGRESSION',ny*2+n,
     *            '//ani98',memw,tek4)
                     jj=-n
                    do j=1,mb
                        ii=-mb
                        jj=jj+n
                        do i=1,n
                              tontuple(2*ny+i)=a(jj+i)
                         end do
          do i=1,ny
               ii=ii+mb
               if(normal.eq.'renorm') then
                 call numnam(tek,numreg(i),n,jreg,isys)
	     tontuple(i)=b(j+ii)*di(i)+tmin(jreg)
                       tontuple(ny+i)=omem1(j+ii)*di(i)+tmin(jreg)
               else
                       tontuple(i)=b(j+ii)
                       tontuple(ny+i)=omem1(j+ii)
               end if
          end do
c
                            call  hfn(ntidw*100+1,tontuple)
                    end do
            end if                        	
        end if
c
c 	CALCULATION OF THE CLASSIFICATION RATES
c
          IF(debug.ge.1.and.qualit.eq.'montec')  THEN
          lines=l
                         do J=1,nipr
                              k=(j-1)*l+1
c
         call rate(omem1,part(k),ccl,ito,L,MB,tel,npp(k),jmode,nt,lines)
                         end do
          end if
c
        if(istg.gt.1) then
C
c ************************* FILLING HISTOGRAM (ntuple) ***************
c
       if (ntupw.eq.1.and.debug.ge.2.and.qualit.eq.'montec') then
               tek(n+1)='netoutpt'
                 call HBOOKN(ntidw*200+1,'NEURAL CLASSIFICATION',n+1,
     *            '//ani98',memw,tek)
                         jj=-n
                         do j=1,mb
                              jj=jj+n
                              do i=1,n
                                   tontuple(i)=a(jj+i)
                              end do
c
                              tontuple(n+1)=omem1(j)
                              call  hfn(ntidw*200+1,tontuple)
                         end do
c
         WRITE(*,'(5X,''TS AND NN OUTPUT NTUPLE  FILLED'')')
       end if
c
        END IF
C
      IF(DEBUG.GE.2.AND.QUALIT.NE.'estima'.AND.NTUPW.EQ.1) THEN
c
c	VISUALISATION OF THE NONLINEAR DECISION SURFACE
c	GENERATION OF PROBES AND CALCULATION OF CORRESPONDING
C 	OUTPUT OF NETWORK WITH PREVIOUSLY OPTIMAZED COUPLINGS
c
c         call vizbon(A,N,MB,L,ITO,B,TEK,SIM,NT,tmin,tmax,adif)
c
c
                   tek(n+1)='NNoutput'		
                   call HBOOKN(ntidw*10,'NEURAL CLUSTER',N+1,
     *            '//ani98',memw,tek)
                    do itau=1,ndel
            call scanpt(probir,n,tmin,tmax,adif,genert,itau)
c
	  call ucopy(probir,q,n)	
          call neural(q,o,tjmem,nodes,leyers,indtot,expmax,debug)
c	
			       	    probir(n+1)=o(1) 					
        		           call HFN(ntidw*10,probir) 			
c			
                    end do
        WRITE(*,'(5X,''NEURAL CLUSTER SCANNING NTUPLE  FILLED'')')
        END IF
c
c        IF(DEBUG.GE.2.AND.QUALIT.EQ.'estima') THEN
c                    ii=-mb+1
c                         do i=1,ny
c                                    ii=ii+mb                     		
c		CALL KOLM2(b(ii),omem1(ii),mb,mb,ZK,PROB)
c	WRITE(NT,'(//5X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''
c     *,2e12.4)') ZK,PROB
c                         end do
c          END IF
c
c******************** CALCULATION OF MEAN ERRORS **********************
C
           if(qualit.eq.'estima') then	
        call errors(oMEM1,b,cc1,cc2,mb,ny,nt)
	   end if
c
C *****************NEURAL NET CLASSIFICATION***************************
C
         ELSE IF(JMODE.EQ.'CLASSIFI'.or.jmode.eq.'EXP') THEN
c
c      changed at 30 07 1997, why =1!?
c
c           L=1
C
       READ (NTJ,'(4I6/10(10I6/))') LEYERS,INDTOT,NNODES,ISHIFT,
     * (NODES(I),I=1,LEYERS)
	READ(NTJ,6) IT
	READ(NTJ,*)(golOLD(I),I=1,L)
	READ(NTJ,6) IT
	READ(NTJ,16)NNORMA
      READ (NTJ,6) IT
      READ(NTJ,*) NITER,CF,SPEED,WSPEED
      READ (NTJ,6) IT
      READ (NTJ,*) (WIGHT(I),I=1,L)
            ITPAR=INDTOT+NNODES
      READ (NTJ,6) IT
      READ(NTJ,'(A6)')QUALIT
C
       READ (NTJ,6) IT
      READ(NTJ,'(A6)')QTYPE
      READ (NTJ,6) IT
      READ(NTJ,'(I5)')NY
      READ (NTJ,6) IT
      READ(NTJ,'(10A10)')(TEK4(I),I=1,NY)
      READ (NTJ,*) (TJ(I),I=1,ITPAR)
C
	if(nnorma.ne.normal) then
       write(isys,'(5x,''Normalisation disagreement !''/2a6)')
     * nnorma,normal
		stop 543
	end if
c
       WRITE (NT,'(5X,''CLASS. (ESTIMATION)  WITH NEURAL NETWORK''/
     *   5x''TRAINED AS FOLLOWING:'')')
       WRITE (NT,'(5X,''N OF RANDOM WALK ITERATIONS, RANDOM STEP'',
     *I8,2X,F7.3)') NITER,CF
c
      WRITE (NT,'(5x,''INITIAL NET SPREAD '',F7.4)')SPREED
      WRITE (NT,'(5X,''ADAPTATION SPEED AND WIEGHT SPEED:'',2F7.4)')
     *SPEED,WSPEED
c
       WRITE (NT,47) search,qualit,qtype,memory,normal,begin,genert
     *  ,stiter,dpoint
     *  ,(NODES(I),I=1,LEYERS)
c
      WRITE(NT,'(5X,''RANDOM SHIFT, ASYMMETRY '',I6,5F15.4)')
     *  ISHIFT, (WIGHT(I),I=1,L)
c
      WRITE(NT,'(5X,''REGRESSAND (S)  '',5a10)')
     * (tek4(i),I=1,ny)
c
               IF(DEBUG.GE.5.) THEN
       WRITE(NT,'(//5X,''IMPLEMENTING THE NETWORK WITH PARAMETERS:'')')
       WRITE(NT,*)(TJ(I),I=1,Itpar)
               END IF
C
      IF(NODES(1).NE.N.and.jmode.eq.'EXP') THEN
      WRITE(ISYS,'(5X,'' NET AND VARIABLES MISMATCH'',6I5)')NODES(1),N
                  STOP 97
      END IF
                if(qualit.eq.'estima') then
                    do i=1,ny
                         IF (NUMREG(I).NE.TEK4(I)) THEN
        WRITE(ISYS,'(5X,'' WRONG REGRESSAND'',2A10)')
     *     NUMREG(I),TEK4(I)
                              STOP 77
                        END IF
                     end do
                end if
               CALL DATIMH(NDATA,NTIME)
               CALL TIMED(TIME)
c        WRITE(NT,'(/2X,''BEFORE CALL NEURAL '',A8,2X,A8)')NDATA,NTIME
        WRITE(*,'(/2X,''BEFORE CALL NEURAL '',A8,2X,A8)')
     * NDATA,NTIME
c
               IF(jmode.eq.'CLASSIFI'.and.qualit.eq.'estima') then
c
                              IF(NODES(1).NE.N-NY) THEN
          WRITE(ISYS,'(5X,''NET AND VARIABLES MISMATCH'',6I5)')
     *   NODES(1),N-NY
                    STOP 57
                              END IF
                    ii=-mp+1
                    do i=1,ny
                         call numnam(tek,numreg(i),n,jreg,isys)
                         cdif(i)=cmax(jreg)-cmin(jreg)
                         ii=ii+mp
                         inout=tek4(i)
                         call varout(t,b(ii),t,n,mp,tek,tek,inout,nt)
                         n=n-1
                              if(normal.eq.'renorm') then
                                call numnam(tek,numreg(i),n,jreg,isys)
        call denorm(b(ii),b(ii),cmin(jreg),cmax(jreg),1,mp)	
c
      write(nt,'(/5x,''CS REGRESSANDS ARE NORMALIZED TO 0-1, ACTUAL
     * VALUES IN:''/50(2X,E12.4,'' < '',A8,'' <'',E12.4/))')
     * (cmin(j),tek4(j),cmax(j),j=1,n)
                              end if
                    end do
               END IF
C
          DO 1145 I=1,MP
                              INDD=(I-1)*N+1
                              CALL UCOPY(t(INDD),Q,N)
c
          if(normal.eq.'renorm'.or.qualit.ne.'estima') then	
          CALL NEURAL(Q,O,TJ,NODES,LEYERS,INDTOT,expmax,debug)
                                    o1(i)=o(1)
c
                                   else
          CALL NEURAL2(Q,O,TJ,NODES,LEYERS,INDTOT,expmax,debug)
c
          iik=-mp
          DO ii=1,ny
                                   iik=iik+mp
                                   o1(I+IIK)=O(ii)
          END DO
                                   end if
C
c         if(t(2).lt.1)then
c               write(isys,'(2i8,7f9.1)')i,mp,t(2)
c               stop 777
c           end if
1145      CONTINUE
c
c           if(i.lt.10)then
c              do indd=1,10*n,n
c                write(isys,'(7f9.1)')(t(ii),ii=indd,indd+n-1)
c              end do
c           end if
c
c******************** CALCULATION OF MEAN ERRORS *****************************
C
          if(qualit.eq.'estima') then	
                   call errors(o1,b,cc1,cc2,mp,ny,nt)
	  end if
C
C
               CALL TIMED(TIME)
               CALL DATIMH(NDATA,NTIME)
c               WRITE(NT,'(/2X,''NEURAL FINISHS AT '',A8,2X,A8,
c     *''       CPU TIME'',E18.8)') NDATA,NTIME,TIME
               WRITE(*,'(/2X,''NEURAL FINISHS AT '',A8,2X,A8,
     *''       CPU TIME'',E18.8)') NDATA,NTIME,TIME
c

          IF(debug.ge.2.and.qualit.ne.'estima')  THEN
c
c	VISUALISATION OF THE EXPERIMENTAL EVENTS
c	CLASSIFICATION BY TRAINED NN
c
                   tek(n+1)='NNoutput'		
                   tek(n+2)='Number'
                   call HBOOKN(ntidw*13,'EXPERIMENTAL EVENTS',N+2,
     *            '//ani98',memw,tek)
c
                    jj=-n
                    do j=1,mp
                              jj=jj+n
c
                              do i=1,n
                                   tontuple(i)=t(jj+i)
                              end do
                              tontuple(n+1)=o1(j)
                              tontuple(n+2)=real(j)
c           if(tontuple(2).le.1.)then
c              write(nt,'(a8,7f9.1)')tek(2),(tontuple(ii),ii=1,n+2)
c           end if				
                              call HFN(ntidw*13,tontuple) 			
		    end do

        WRITE(*,'(5X,''EXPERIMENTAL EVENTS NTUPLE  FILLED'')')
c
c
c 	CALCULATION OF THE CLASSIFICATION RATES
c
          lines=1
c
                         do J=1,nipr
                              k=(j-1)*l+1
        call rate(o1,part(k),ccl,itocon,L,MP,tel,npp(k),jmode,nt,lines)
                         end do
c
         END IF
c
          IF(debug.ge.6.and.qualit.eq.'estima')  THEN
                               ii=-mp+1
                              do i=1,ny
                                ii=ii+mp
        CALL KOLM2(b(ii),o1(ii),mp,mp,ZK,PROB)
	WRITE(NT,'(2X,''KOLMOGOROV TEST VALUE AND CONFIDENCE:''
     *,2F15.10)') ZK,PROB
                              end do
          END IF
C
          if(qualit.eq.'estima')then
c
               do i=1,ny
                    tek3(n+ny+i)= tek4(i)
                    tek3(n+i)=tek4(i)
                    lennam=index(tek3(n+i),' ')
c
                    if(lennam.le.4) then
                         tek3(n+i)(lennam:lennam+3)='estm'
                    else
                         tek3(n+i)(5:8)='estm'
                    end if
c
               end do
c
               do i=1,n
                         tek3(i)=tek(i)
               end do
c
               if(jmode.eq.'CLASSIFI') then
c
                    if(ntupw.eq.1) then
                 call HBOOKN(ntidw*100+1,'NEURAL REGR. CONTROL',ny*2+n,
     *            '//ani98',memw,tek3)
                         jj=-n
c
                       do j=1,mp
                              ii=-mp+1
                              jj=jj+n
c
                              do i=1,n
                                   tontuple(i)=t(jj+i)
                              end do
cc
                              do i=1,ny
                                   ii=ii+mp
c
                                   if(normal.eq.'renorm') then
                         call numnam(tek,numreg(i),n,jreg,isys)
                         tontuple(n+ny+i)=b(j+ii)*cdif(i)+cmin(jreg)        		
                         tontuple(n+i)=o1(j+ii)*cdif(i)+cmin(jreg)
                                   else
                         tontuple(n+ny+i)=b(j+ii)
                         tontuple(n+i)=o1(j+ii)
                                   end if
                              end do
                         call  hfn(ntidw*100+1,tontuple)
                       end do
c
               end if
c
c-Aro-              if (debug.ge.4.) then
               if(debug.ge.4.and.access.eq.'SEQUENTAL') then
	            ito(1)=MP
	            ito(2)=0
                    do i=1,nban
c                              amin(I)=-1.e10
c                              amax(I)=1.e10
                        numb(i)=i
                    end do	
c
	            mnewc=0
        CALL FREAD(t,wlum,RB,Icon,NBAN,MNEWC,TEK,DEBUG,ictot,m22,m33,
     *  NBAN,NUMB,AMIN,AMAX,IcTOT,FORMAT1,IFORM1,IcSH,NT,ACCESS,tek4)
c
         INOUT= 'E0estim'
        CALL VARIN(t,O1,a,NBAN,MP,TEK,TEK4,INOUT,NT,ISYS)
                    lennam=index(IDUM,' ')
                    IDUM(lennam:lennam+2)='ALL'
        CALL FWRITE(a,wlum,IDUM,Icon,MP,Nban+1,TEK4,
     *  FORMAT1,IFORMW,nt)
c
c-Aro- changed to have all variables in IDUM file; see above lines.
c Yet for one regressand
c            CALL VARIN(t,O1,a,N,MP,TEK,TEK,TEK3(N+1),NT,ISYS)
c
c                              if(NY.EQ.2) then
c            CALL VARIN(a,O1(mp+1),t,N+1,MP,TEK,TEK,TEK3(N+2),NT,ISYS)
c            CALL UCOPY(t,a,(n+2)*mp)
c                              end if
c
c            CALL VARIN(a,b,t,N+NY,MP,TEK,TEK,TEK4(1),NT,ISYS)
c            CALL UCOPY(t,a,(n+ny+1)*mp)
c
c                              if(NY.EQ.2) then
c            CALL VARIN(t,b(mp+1),a,N+NY+1,MP,TEK,TEK,TEK4(2),NT,ISYS)
c                              end if
c
c   WRITING CONTROL SAMPLE + ESTIMATED ENERGY AND MASS + TRUE ENERGY AND MASS
c	
c        CALL FWRITE(a,wlum,IDUM,Icon,mp,N+2*NY,TEK3,FORMAT1,IFORMW,nt)
c-Aro- Now the end if of (debug.ge.4)
                         end if
c
          else if (jmode.eq.'EXP') then
c
                    if(ntupw.eq.1) then
                 call HBOOKN(ntidw*100+1,'NEURAL REGR. EXP',ny+n,
     *            '//ani98',memw,tek3)
                          jj=-n
                         do j=1,mp
                              jj=jj+n
c
                              do i=1,n
                                   tontuple(i)=t(jj+i)
                              end do
c
                              ii=-mp+1
                              do i=1,ny
                                    ii=ii+mp
                                   tontuple(n+i)=o1(j+ii)
                              end do
                              call  hfn(ntidw*100+1,tontuple)
                         end do
                    end if
c
                              if (debug.ge.4.) then
                                   ii=-mp+1
                                   do i=1,ny
                                        ii=ii+mp
               CALL VARIN(t,O1(II),a,N,MP,TEK,TAGS,TEK3(N+I),NT,ISYS)
                                   end do
c
c   WRITING EXP. SAMPLE + ESTIMATED ENERGY AND MASS
c	
        CALL FWRITE(a,wlum,IDUM,Icon,mp,N+NY,TEK3,FORMAT1,IFORMW,nt)
                              end if
          end if
c
       end if
c ************************** BOOKING OF SELECTED AND REJECTED EVENTS ***********
c *********** PREPARATION OF NUMBERS OF EVENTS TO BE WRITE IN IDUM FILE ****
c
       if (jmode.eq.'EXP'.or.jmode.eq.'CLASSIFI')then
           if(debug.ge.2.and.qualit.ne.'estima') then
       tek(n+1)='NNoutput'
        tek(n+2)='Number'
       call HBOOKN(ntidw*10+3,'Selected events',N+2,
     *            '//ani98',memw,tek)
c
              do i=1,mp
                        indd=(i-1)*n+1
                    if(o1(i).le.dpoint) then
                                     ievent= ievent+1
                                    if(ntupw.eq.1) then	
                     call ucopy(t(indd),tontuple,n)
                     tontuple(n+1)=o1(i)
                     tontuple(n+2)=real(i)
c
                     call HFN(ntidw*10+3,tontuple)
                                   end if
                    end if        	    	
              end do
c	
               if(debug.ge.4.and.access.eq.'SEQUENTAL') then
	            ito(1)=MP
	            ito(2)=0
                    do i=1,nban
c                              amin(I)=-1.e10
c                              amax(I)=1.e10
                        numb(i)=i
                    end do	
c
	            mnewc=0
c	
        CALL FREAD(t,wlum,RB,Icon,NBAN,MNEWC,TEK,DEBUG,ictot,m22,m33,
     *  NBAN,NUMB,AMIN,AMAX,IcTOT,FORMAT1,IFORM1,IcSH,NT,ACCESS,tek4)
c
         INOUT= 'NETOUTPT'
        CALL VARIN(t,O1,a,NBAN,MP,TEK,TEK4,INOUT,NT,ISYS)
                    lennam=index(IDUM,' ')
                    IDUM(lennam:lennam+2)='ALL'
        CALL FWRITE(a,wlum,IDUM,Icon,MP,Nban+1,TEK4,
     *  FORMAT1,IFORMW,nt)
c
                    WRITE(NT,807)Icon,mp,FORMAT1,IDUM
               end if
       end if
       end if
c
C*******************BHATACHARIA DISTANCE CALCULATION*****************
C
      ELSE IF(JMODE.EQ.'BHATOT'.and.l.eq.2)THEN
C
        CALL MISR1(A,R2,R,U,N,ITO,MB,L)
        CALL BHA(R,R2,E2,E3,E,RBHA,RMAH,COR,N,L,NT)
      DO 965 I=1,N
      J=(I-1)*N
        Y(I)=RBHA(I)
        Y(I+N)=EXP(-RBHA(I))/2.
        YCC=1.-4.*Y(I+N)**2
      IF(YCC.LE.1.E-30)THEN
         WRITE(8,'(5X,''ATTENSION YCC.LE-'',E20.4)')YCC
         YCC=1.E-30
      END IF
       Y(I+2*N)=0.5*(1.-SQRT(YCC))
       Y(I+3*N)=RMAH(I)
       Y(I+4*N)=COR(I)
965    CONTINUE
       CALL PRT(Y,N,5,1,C21,TEK,TEB,NT,jmode)
C     CALL HILLAS(A,U,HILL,N,ITO(1),MB)
C       CALL PRT(HILL,1,N,1,C20,TE,TEK,NT,jmode)
C
c
c  ******** SELECTION OF BEST VARIABLES ACCORDING TO BHATACHARIA DISTANCE *****
c
      ELSE IF(JMODE.EQ.'BHATA'.OR.JMODE.EQ.'MAHALO'.OR.JMODE.
     *EQ.'CORR'.and.l.eq.2) THEN
C
      DO 3005 J=N, INTDIM+1,-1
        DO   I=1,j
c        write(nt,'(''before out n,intdim,i,j'',7i7)') n,intdim,i,j
          CALL OUT(A,T,MB,J,I)
c        write(nt,'(''after out n,intdim,i,j'',7i7)') n,intdim,i,j
          CALL MISR1(T,R2,R,U,J-1,ITO,MB,L)
          CALL BHAFAS(R,R2,E,E1,E2,E3,BHATDI,RMAHDI,CORDI,J-1)
            IF(JMODE.EQ.'BHATA') THEN
               RB(I)=BHATDI
           ELSE IF(JMODE.EQ.'MAHALO') THEN
              RB(I)=RMAHDI
           ELSE IF(JMODE.EQ.'CORR') THEN
             RB(I)=CORDI
          END IF
       END DO
        LRBMAX=LVMAX(RB,J)
        WRITE(NT,2007)JMODE,TEK(LRBMAX),(RB(I),I=1,J)
        TEK3(N-J+1)=TEK(LRBMAX)
        CALL OUT(A,A,MB,J,LRBMAX)
        NEW=0
        DO 5005 II=1,J
        IF(II.NE.LRBMAX)THEN
        NEW=NEW+1
        TEK(NEW)=TEK(II)
        END IF
5005    CONTINUE
2007    FORMAT(//2X,A8,'CRITERION, FEATURE  ',A10, '- OUT, DISTANCES'/
     * 5(2X,10F8.3/))
3005    CONTINUE
        WRITE(NT,4007)INTDIM,JMODE,(TEK(I),I=1,INTDIM)
4007   FORMAT(//2X,'INTRINSIC DIMENSION',I5,2x,A10,' DISTANCE CUT' /
     *2X,'BEST (SURVEVED) FEATURES:'/2X,20A10)
        WRITE(NT,3007)INTDIM,(TEK3(I),I=1,N-INTDIM)
3007   FORMAT(//2X,'INTRINSIC DIMENSION',I5/
     *2X,'WORST FEATURES:'/2X,20a10)
C
C*************************ONE DIMENSIONAL ANALYSIS*****************************
C
      ELSE IF(JMODE.EQ.'ONE-DIMEN'.and.L.eq.2) THEN
C
        CALL MISR1(A,R2,R,U,N,ITO,MB,L)
        N1=1
        DO 105 I=1,N
        IB=(I-1)*(N+1)+1
        E3M(1)=1./R(IB)
        E3M(2)=1./R(IB+N*N)
        E1M(1)=R(IB)
        E1M(2)=R(IB+N*N)
        E2M(1)=R2(I)
        E2M(2)=R2(I+N)
      CALL STUD(E2M,E1M,CC1,CC2,N1,MB,L,ITO(1),ITO(2))
                X(I)=CC1(1)
        X(I+3*N)=CC2(1)
C
        KK=0
        DO 115 K=I,ITO(1)*N,N
        KK=KK+1
        UA(KK)=A(K)
115     CONTINUE
        KK=0
        DO 125 K=ITO(1)*N+I,MB*N,N
        KK=KK+1
        UA(ITO(1)+KK)=A(K)
        WLUM(KK)=A(K)
125     CONTINUE
c      ********************* ATTENSION !!!! KOLM2 IS SORTING INPUT ARRAYS ************************
c      KOLMOGOROV-SMIRNOF TEST FOR ESTIMATED and "TRUE" ENERGY DISTRIBUTIONS
c
       CALL KOLM2(UA,WLUM,ITO(1),ITO(2),ZK,PROB)
         X(I+N)=ZK
         X(I+4*N)=PROB
C
         CALL UTEST(UA,WLUM,ITO(1),ITO(2),UT,ZU,IERR)
              X(I+2*N)=ABS(ZU)
        X(I+5*N)=FG(ZU)
105     CONTINUE
        CALL PRT(X,N,3,2,C6,TEK,TE,NT,jmode)
C
C*****************BAYESSIAN CLASSIFICATION**************************************
C
       ELSE IF(JMODE.EQ.'CLASSIFICATION') THEN
C
         IF(STATUS.eq.'DENCURVE') THEN
c
                     ij=-n+1
                     mp=ndel
		     itocon(1)=ndel
                    do itau=1,mp
                      ij=ij+n
	  if(sim.eq.1.) then
	       do i=1,n
	            cc1(i)=amin(numb(i))
		    cc2(i)=amax(numb(i))
		    di(i)=cc2(i)-cc1(i)
	       end do
	  else
	       do i=1,n
	            cc1(i)=tmin(i)
		    cc2(i)=tmax(i)
		    di(i)=adif(i)
	       end do
	  end if	
c	
            call scanpt(T(ij),n,cc1,cc2,di,genert,itau)
c     	
                    end do
         END IF
C
        CALL PARZ(A,T,U,PST,DEN,R,DI,N,ITO,MB,L,MP,KCL,m22,del,nei,
     *  WLUM,AP,IAP,F,DETER,DS,DFM,JMODE,DEBUG,JDEN,NT,
     *  expmax,strang)
C
C ********** PLOTTING (PAW) NONPARAMETRIC DENSITY ESTIMATES *********
C
        IF(STATUS.eq.'DENCURVE') THEN
c
               KCLS= (kcl+2)*l
	            DO I=1,N
                         WRITE(TAGS(kcls+i), '(''RANGE'',I1)') I
                    END DO
		     tags(kcls+n+1)='GAUSS'
                     ii=0
                    DO J=1,kcl+2
                         DO I=1,L
                              ii=ii+1
                              WRITE(TAGS(ii), '(''PARMKL'',2I1)') J,I
                         END DO
                    END DO
c
               do J=1,iap
                              k=(j-1)*l+1
       call HBOOKN(ntidw*10+j,'DENSITY ESTIMATES',kcls+n+1,'//ani98',
     *                        MEMW,tags)
c
               CALL DENFIT(DS,T,PF,ap(k),N,MP,L,KCL,NTIDW*10+j)
c
               end do
	       IF(MP.LE.10) THEN
      	            CALL PRT(DS,MP,L,KCL+2,C15,men,MEN,NT,jmode)     	
	            CALL PRT(B,MP,1,1,C35,men,MEN,NT,jmode)     	
               END IF
c
               do J=1,iap
                              k=(j-1)*l+1
c
               WRITE(IT, '(''BAYESIAN CLUSTER TIGHTNESS='',f5.2)')ap(k)
                    call HBOOKN(ntidw*20+j,IT,n,'//ani98',MEMW,tek)
                    CALL DENCURVE2(DS,T,AP(K),MP,N,L,KCL,pst,NTIDW*20+j)
               end do
C
       ELSE
C	
      	    CALL LOGLI(DS,B,RDFM,MP,L,KCL+2,STRANG,sim)
c
	  CALL MISR1(B,R2,R,WLUM,KCL+2,ITOCON,MP,1)
c
          WRITE(NT, '(/5x,''LOG.- RATIO TRUNCATED TO  '',f7.1)')sim
	                 CALL PRT(R2,1,KCL+2,1,C49,TEL,FF,NT,jmode)
c                          WRITE(NT,'(//2X,''LOG.RATIO VARIANCES''/
c     * 11F7.3)') (sqrt(R(I)),I=1,(KCL+2)**2,KCL+3)
c
     	    CALL PRT(RDFM,L,KCL+2,1,C9,TEL,FF,NT,jmode)
c
C ********** PLOTTING (PAW) CLASSIFICATION RESULTS *********
C

                     tek(n+1)='class'
		     tek(n+2)='ratioad'
		     tek(n+3)='ratiofr'
c
       call HBOOKN(ntidw*10,'CLASSIFICATION RESULTS',n+3,'//ani98',
     *             MEMW,tek)
c
                do j=1,mp
c		
                     do jj=1,n
                         tontuple(jj)=t(jj+(j-1)*n)
                     end do
c
c     ******** CLASSIFICATION RESULTS ONLY FOR THE CASE OF PARZ DENSITY ESTIMATES  !!!!!!!! **********
c		
		     tontuple(n+1)=real(m22(j))
		     tontuple(n+2)=b((kcl+2)*j)
		     tontuple(n+3)=b(1+(kcl+2)*(j-1))
c		
                     call HFN(ntidw*10,tontuple)
                end do
c
           irr(1)=mp
	   mmax=mp
           ii=1
	   kk=1
          DO I=1,L
	  call ucopy(t,b,mp*n)
	  call ucopy(wlum,dp,mp)
	  call ucopy(a(ii),b(mp*n+1),ito(i)*n)
	  call ucopy(u(kk),dp(mp+1),ito(i))
c	
	  irr(2)=ito(i)
	  if(irr(2).ge.mmax)then
	       mmax=irr(2)
	  end if
c
c CALCULATION OF MAHALANOBIS DISTANCE AND BAYS ERROR FOR THEORETICAL VALUES
c
	  CALL MISR1(B,SM,SV,DP,N,IRR,MMAX,L)
c
           CALL BHAFAS(SV,SM,E,E1,E2,E3,RMAH(2),RMAH(1),RMAH(3),N)
	       SQRMAH=SQRT(RMAH(1))	
	       RMAH(4)=FG(-SQRMAH/2.)
	       CALL PRT(RMAH,1,4,1,C45,MEN(i),TEB,NT,jmode)
c	
             kk=kk+ito(i)
             ii=ii+ito(i)*n
	  END DO		
C
      DO 1645 I=1,IAP
       ISHI=L*L*(KCL+2)*(I-1)+1
	ISHI1=(I-1)*L+1
	ISHI2=I*L
        write(nt,'(/15x,''COST FUNCTION VALUES- '', 5f6.2)')
     * (ap(j), j=ishi1,ishi2)
	 CALL PRT(dfm(ishi),L,KCL+2,1,C1,TEL,FF,NT,jmode)
1645   CONTINUE
C
        IF(PORT.EQ.'RECONSTRUCT') THEN
        REWIND NTEMP
48      CONTINUE
        READ(NTEMP,27,END=28)NREAD, MTOTOB,KCLOB,LOB,IAPOB,ishic,RJMODE,
     *   RJDEN,RTSNAM,RTEK1,RDFF,
     *  (RDFM(I),I=1,LOB*(KCLOB+2)*LOB*IAPOB)
C
        IF(NREAD.NE.N) THEN
        WRITE (ISYS,'('' NUMBER OF VARIABLES  MISMATCH''
     1  /2i3)') N, NREAD
        GO TO 48
        END IF
	IF(ishic.NE.ishcod) THEN
        WRITE (ISYS,'('' TRAINING SHIFT MISMATCH''
     1  /2i3)') ishic, ishcod
        GO TO 48
        END IF
c	
       IF(RTSNAM.NE.TSNAME(1)) THEN
        WRITE (ISYS,'('' ERROR READING *TEM :TRAINING MISMATCH''
     1  /2A50)') TSNAME(I),RTSNAM
        GO TO 48
        END IF
C
        IF(IAPOB.NE.IAP) THEN
        WRITE (ISYS,'('' PRICES MISMATCH''
     1  /2i3)') IAPOB,IAP
        GO TO 48
        END IF
        IF(JDEN.NE.RJDEN) THEN
        WRITE (ISYS,'('' ERROR READING *TEM :EST. MODE MISMATCH''
     1  /2A20)') JDEN,RJDEN
        GO TO 48
        END IF
        IF(RTEK1(1:N*8).NE.WTEK1(1:N*8)) THEN
        WRITE (ISYS,'('' ERROR READING *TEM :VARIABLES MISMATCH''
     1  /2A20)') RTEK1,WTEK1
C
                GO TO 48
        END IF
        IF(RDFF(1:kcl*4).NE.DFF(1:kcl*4)) THEN
        WRITE (ISYS,'('' ERROR READING *TEM :KERNELS MISMATCH''
     1  /2A20)') RDFF,DFF
                GO TO 48
        END IF
        IF(RJMODE.NE.'ONE-LEAVE-OUT-') THEN
        WRITE (ISYS,'('' ONE-LEAVE-OUT TEST REQUIRED'')')
                GO TO 48
        END IF
       IF(MTOTOB.NE.MB) THEN
       WRITE (ISYS,'('' ERROR READING *TEM :TRAIN.SIZE MISMATCH''
     1  /2I7)') MTOTOB,MB
                GO TO 48
        END IF
C
       DO 745 I=1,IAP
        ISHI=L*L*(KCL+2)*(I-1)+1
	ISHI1=(I-1)*L+1	
	ISHI2=I*L
c
c ************** Variant only for 2 classes **************************
        IF(L.EQ.2.AND.DEBUG.GE.2) then
c         CALL PRT(rdfm,L,KCL+2,L,C4,TEL,FF,NT,jmode)
         CALL DFMCL(RDFM(ISHI),GOB,ERROB,AP(ISHI1),L,L,KCLOB+2)
         CALL DFMCL(DFM(ISHI),G,ERR,AP(ISHI1),L,1,KCL+2)
         CALL PFERR(RDFM,ERROB,DFM,ERR,PF,PFER,PFERM,PFERS,KCL+2,
     *   N,L,TEL,FF,NT,c9,c4,jmode)
         CALL PRT(PF,1,KCL+2,1,C11,TEL,FF,NT,jmode)
c         CALL PRT(PFERM,1,KCL+2,1,C13,TE1,FF,NT,jmode)
C         CALL PRT(PFER,1,KCL+2,1,C12,TEL,FF,NT,jmode)
        END IF
c ********************************************************************
c
        write(nt,'(/15x,''COST FUNCTION VALUES- '', 5f6.2)')
     * (ap(j), j=ishi1,ishi2)
C
	call prfer(rdfm(ishi),dfm(ishi),r3,r2,pf,c0,cc,rr,dr3,dc0,L
     *  ,kcl+2,nt,debug,IRR)
        CALL PRT(cc,1,KCL+2,1,C51,TEL,FF,NT,jmode)
        CALL PRT(PF,L,KCL+2,1,C11,TEL,FF,NT,jmode)
745    CONTINUE
C
         END IF
       END IF
C
C********************BAYESSIAN LEARNING*****************************************
C
      ELSE IF (JMODE.EQ.'ONE-LEAVE-OUT-') THEN
C
        CALL PARZ(A,A,U,PST,DEN,R,DI,N,ITO,MB,L,MB,KCL,m22,del,nei,
     * WLUM,AP,IAP,F,DETER,DS,DFM,JMODE,DEBUG,JDEN,NT,
     * expmax,strang)
C
        IF(N.EQ.1.AND.ITO(1).LE.8) THEN
   	      CALL PRT(A,N,ITO(1),L,C18,MEN,IDEN,NT,jmode)
        END IF
C
       DO 645 I=1,IAP
       ISHI=L*L*(KCL+2)*(I-1)+1
	ISHI1=(I-1)*L+1
	ISHI2=I*L
        CALL DFMCL(DFM(ISHI),G,ERR,AP(ISHI1),l,l,KCL+2)
        write(nt,'(/15x,''COST FUNCTION VALUE- '', 5f6.2)')
     * (ap(j), j=ishi1,ishi2)
c
         CALL PRT(dfm(ishi),L,KCL+2,L,C4,TEL,FF,NT,jmode)
         CALL PRT(G,L,KCL+2,1,C5,TEl,FF,NT,jmode)
	 CALL PRT(ERR,1,KCL+2,1,C3,TEl,FF,NT,jmode)
645    CONTINUE
C
C
C ***** FIND END OF FILE NTEMP  *****
C
        do I999=1,99999
          read(ntemp,27,end=448)NOB,MTOTOB,KCLOB,LOB,IAPOB,ishic,RJMODE,
     *   RJDEN,TSNAM,RTEK1,RDFF,
     *  (RDFM(I),I=1,LOB*(KCLOB+2)*LOB*IAPOB)
        end do
448        continue
C
        WRITE(NTEMP,27)N, MB,KCL,L,IAP,ishcod,JMODE,JDEN,
     *TSNAME(1),WTEK1,DFF,(DFM(I),I=1,L*(KCL+2)*L*IAP)
C
        ELSE
C
       WRITE(ISYS,'(''This jobe mode did not support multiclass case''/
     *'' OR UNKNOWN JOBE MODE  '',A40)') JMODE
        STOP 26
       END IF
C
            go to 68
28          WRITE(NT,137) STATUS,JMODE,JDEN
68           continue
C
               CALL TIMED(TIME)
C
             CALL DATIMH(NDATA,NTIME)
             WRITE(NT,'(/2X,''RUN FINISHS AT '',A8,2X,A8,
     *'' CPU TIME'',E12.4)') NDATA,NTIME,TIME
            WRITE(*,'(/2X,''RUN FINISHS AT '',A8,2X,A8,
     *'' CPU TIME'',E12.4)') NDATA,NTIME,TIME
c
c *********************** ntuple and files CLOOSING ****************************
c
       if(ntupw.gt.0.and.debug.ge.2.and.jmode.ne.'DIMDIM'
     *    .and.jmode.ne.'CVM') then
         call HROUT(0,ICYCLE,' ')
         call HREND('ani98')
       end if
         CLOSE (NT)
         CLOSE (NTEMP)
         CLOSE (NTJ)
C
6       FORMAT(A80)
16     format(a6)
27      FORMAT(1X,6i6/2(1x,A50/)/3(1X,A100/)/60(1X,6F10.5/))
57      FORMAT(70('*')/,(10F10.4/10F10.4)/,70('*'))
77      FORMAT(10(10F7.3/))
137     FORMAT(//2X,'NO RECORD FROM *.TEM MATCH GIVEN JOB'/
     *5X,'STATUS = ',A12,' OPERATION MODE = ',A14/
     *5X,'DENSITY ESTIMATION MODE:',A12)
c
147     FORMAT(//2X,'END OF MULTIVARIATE DATA ANALYSIS '/
     * 5X,'JOB STATUS = ',A12,' OPERATION MODE = ',A14/
     *5X,'DENSITY ESTIMATION MODE:',A12)
c
507   FORMAT(//64(1H*)//5X,'NEW SAMPLE WAS WRITENIN DATA BASE'/
     *5X,'SAMPLE NAME:',A12,'# OF EXTRACTED EVENTS',2I7/
     *,50(2X,E12.4,' < ',A8,' <',E12.4))
607   FORMAT(/5X,'TRAINING SAMPLE IS SELECTED'/
     *5(5x,i3,' class -',I7,' EVENTS FROM ',I7/))
707   FORMAT(//78(1H*)//5X,'CONTROL SAMPLE SELECTED'/
     *2X,I7,' EVENTS FROM ',I7)

907   FORMAT(/5X,' SUPERIMPOSED BOUNDARIES: '/
     *,50(2X,E12.4,' < ',A8,' <',E12.4/))
807	   FORMAT(//64(1H*)//2X,'NEW FILE IS CREATED:FROM  SAMPLE '/
     *5X,a50,i7,' EVENTS ARE WRITEN  '/
     *5X,A50,' FORMAT '/
     *5X,A50,' NAME')
C
        STOP
        END
