*CMZ :          20/06/97  18.26.36  by  Ararat Vardanyan
*-- Author :
C
C
      FUNCTION IDIM1 (P,M,N,L,NT,O,D,DD,M2,RNN)
C        AN ALGORITHM TODETERMINE THE DIMENSIONALITY OF A SET OF
C.        POINTS LOCATED IN THE NEIGHBORHOOD OF AN ORIGIN. ASSUMES
C.    HOMOGENEOUS DISTRIBUTION IN THE SUBSPACE.(MODIFIED- NO ASSUMPT)
C.            ORIGIN =  B.BRAAMS  '74, STATUS = TESTED.
C        REVISED = A.CHILINGARIAN '87, STATUS = UNDER TESTING
C
C *** A PROCEDURE TO DETERMINE, UNDER RESTRICTIVE ASSUMPTIONS, THE
C *** DIMENSIONALITY IN THE NEIGHBORHOOD OF A CERTAIN POINT O, OF THE
C *** SUBSPACE IN WHICH A SET OF POINTS IS LOCATED.
C ***
C *** METHOD: IF N POINTS ARE DESTRIBUTED HOMOGENEOUSLY, (NOT ONLY
C *** HOMOGENEOUSLY) IN AN I-DIMENSIONAL SUBSPACE, AND O IS PART OF
C *** THAT SUBSPACE,AND KNN  ENOTES THENUMBER OFPOINTS WITHIN A
C *** HYPERSPHERE OF RADIUS R AND CENTER IN O, THEN
C *** KNN   ==APPR==  C*R**I. THUS LOG(KNN )  ==APPR==  LOG(C)+I*LOG(R)
C *** I CAN BE COMPUTED BY MAKING A LEAST SQUARES FIT.
C *** POINTS CLOSEST TO O, WHICH HAVE VERY SMALL R, AND VERY LARGE
C *** LOGNR), WOULD MAKE A DISPROPORTIONATE CONTRIBUTION TO THE FINAL
C *** RESULT. FOR THIS REASON, ALL POINTS ARE GIVEN A WEIGHT, EQUAL TO
C *** THEIR DISTANCE TO O.
C ***
C *** IMPLEMENTATION: THE USER SPECIFIES A SET OF POINTS,
C *** TO DETERMINE DIMENSIONALITY DISTRIBUTION.
C ***
C *** P IS AN (M,NN) ARRAY, CONTAINING A SET OF N POINTS IN M DIMENSIONS
C *** D MUST BE A VECTOR WITH AT LEAST 2*N-1 COMPONENTS. ITS CONTENTS
C *** WILL NOT BE MEANINGFULL TO THE USER AFTER A CALL TO IDIM.
C *** L IS PROVIDED BY THE USER. IN FINDING THE DIMENSIONALITY, IDIM
C *** WILL CONSIDER THE L POINTS WITH SMALLEST D.
C *** DD WILL CONTAIN, ON RETURN, THE DISTANCES TO O OF THE L POINTS
C *** CLOSEST TO O. IT IS A VECTOR.
C *** A(1)IS THE COMPUTED DIMENSIONALITY; THIS IS A FLOATING-POINT NUMBE
C *** A(2) IS MODIFIED DIMENSIONALITY
C *** M2 IS FOR USE BY IDIM; ITS CONTENTS WILL NOT BE MEANINGFULL UPON
C *** RETURN. M2 IS A VECTOR WITH AT LEAST 2*N-1 COMPONENTS.
C *** BIGST IS USED AS 'INFINITY'. IT IS MACHINE-DEPENDANT.  =1.E290 CDC
C ***
C *** ALL THE INPUT PARAMETERS ARE LEFT UNCHANGED BY IDIM.
C ***
C ***
      DIMENSION P(M,N),O(M),DD(L),D(N*2),M2(N*2),RNN(L)
      DATA BIGST/1.0E19/
C *** N POINTS IN M DIMENSIONS
      KCL=L-1
      IF (M.LT.1)    GO TO 140
      IF (L.LT.2)    GO TO 170
      IF (N.LT.L)    GO TO 180
      AN = N
C ***
C *** WE NOW LOAD INTO THE UPPER HALF OF D FOR EACH POINT THE DISTANCE
C *** BETWEEN THAT POINT AND O. WE SET M2 FOR USE IN THE SORTING PHASE.
C ***
      DO 50 I = 1,N
      IP = I+N-1
      M2(IP) = IP
      D(IP) = (P(1,I)-O(1))**2
      IF (M.EQ.1)    GO TO 50
      DO 40 J = 2,M
      DEL=P(J,I)-O(J)
   40 D(IP) = D(IP)+DEL*DEL
   50 D(IP) = SQRT(D(IP))
C ***
C *** WE SORT INTO DD THE SMALLEST L ELEMENTS AMONG THE N
C *** ENTRIES THAT WE HAVE IN THE UPPER HALF OF D.
C *** THE METHOD USED IS A TRANSLATION OF TREESORT 1
C *** (ALGORITHM 143, C.A.C.M)
C ***
      DO 70 II = 2,N
      I = N+1-II
      IF (D(2*I).GT.D(2*I+1))    GO TO 60
      D(I) = D(2*I)
      M2(I) = M2(2*I)
      GO TO  70
   60 D(I) = D(2*I+1)
      M2(I) = M2(2*I+1)
   70 CONTINUE
      DO 100 J = 1,L
      DD(J) = D(1)
      I = M2(1)
      D(I) = BIGST
   80 I = I/2
      IF (I.EQ.0)    GO TO 100
      IF (D(2*I).GT.D(2*I+1))    GO TO 90
      D(I) = D(2*I)
      M2(I) = M2(2*I)
      GO TO  80
   90 D(I) = D(2*I+1)
      M2(I) = M2(2*I+1)
      GO TO  80
  100 CONTINUE
C ***
C *** TREESORT FINISHED.
C ***
C     WRITE(NT,'(10F7.3)')DD
      DO 65 I=1,KCL
65    DD(I)=DD(I+1)
C ***
      SXX = 0.
      SXY = 0.
      SX = 0.
      SY = 0.
      W = 0.
C *** TO EACH POINT, A WEIGHT IS ASSIGNED, EQUAL TO ITS DISTANCE TO O.
      DO 130 I = 1,KCL
      X = ALOG(FLOAT(I))
      IF(DD(I).LE.1.E-19) THEN
      DD(I)=1.E-19
      END IF
      Y = ALOG(DD(I))
      SX = SX+DD(I)*X
      SXX = SXX+DD(I)*X*X
      SXY = SXY+DD(I)*X*Y
      SY = SY+DD(I)*Y
  130 W = W+DD(I)
      IDIM1 = (W*SXX-SX*SX)/(W*SXY-SX*SY)
      RETURN
C ***
  140 PRINT 2000,M

      RETURN
  150 PRINT 2010,N
      RETURN
  160 PRINT 2020,K
      RETURN
  170 PRINT 2030,L
      RETURN
  180 PRINT 2040,N,L
C ***
 2000 FORMAT(38H ERROR IN FUNCTION IDIM: M LESS THAN 1,I5)
 2010 FORMAT(38H ERROR IN FUNCTION IDIM: N LESS THAN 2,I5)
 2020 FORMAT(38H ERROR IN FUNCTION IDIM: K LESS THAN 1,I5)
 2030 FORMAT(38H ERROR IN FUNCTION IDIM: L LESS THAN 2,I5)
 2040 FORMAT(38H ERROR IN FUNCTION IDIM: N LESS THAN L,I5,I5)
C ***
      RETURN
      END
