*CMZ :          22/01/97  16.51.11  by  Ararat A. Vardanyan
*-- Author :    Eugene Sevinian   10/24/96
C-----------------------------------------------------------------------
C NTROWR reads a RWN HBOOK-Ntuple. Cuts may be applied on each parameter.
C It is possible to skip events or to read them in a selected order.
C A subset of parameters may be chosen. Those events that were chosen
C to be read and survived the cuts will be returned, including the
C corresponding weights and the order after which they were read.
C Print debug info if IDEBUG = -7

      SUBROUTINE NTROWR(NTID,LUNHB,LUNMSG,IDEBUG,DSNHB,NPARMX,NEVTMX,
     +                NPARAC,NEVTAC,NEVRD,NEVTSK,LEVTOR,NUMBEV,CHPARS,
     +                EVTORI,EVTORO,PARCTL,PARCTU,EVTWT,EVENTS)

* in the following * and C indicate INPUT and OUTPUT, respectively.

*...NTID  : Identifier (ID) of the Ntuple to be read
*...LUNHB : HBOOK - Unit
*...LUNMSG: Unit for messages according to verbose-level DEBUG
*...IDEBUG : verbose-level (0...5)
*...DSNHB : HBOOK - Filename
*...NPARMX: Maximum number of parameters
*...NEVTMX: Maximum number of events
*...NPARAC: Actual number of parameters (to be used)
*...NEVTAC: Actual number of events (to be read)
C...NEVRD : Nr of events actually read
*...NEVTSK: Number of events to be skipped
*...LEVTOR: FLAG: read events in selected order? (=> .TRUE.)
*...NUMBEV: Vector of parameter-numbers to be used
C...CHPARS: CHAR*8-Vector of corresponding parameter-descriptors (names)
*...EVTORI: Vector containing Event ORder to use if LEVTOR==.TRUE.
C...EVTORO: Vector containing Event ORder AFTER reading (and maybe cuts)
*...PARCTL: Vector containing LOWER cuts on parameters
*...PARCTU: Vector containing UPPER cuts on parameters
C...EVTWT : Vector containing event WEIGHTS
C...EVENTS: Array containing the events in terms of the chosen parameters


      DIMENSION EVENTS(NPARAC,NEVTMX), EVTWT(NEVTMX)
      DIMENSION PARCTL(NPARMX), PARCTU(NPARMX)
      DIMENSION EVTORI(NEVTMX), EVTORO(NEVTMX)
      DIMENSION NUMBEV(NPARMX)
      LOGICAL   LCUT, LEVTOR
      CHARACTER*50 DSNHB

      PARAMETER (NBIG=100)
      CHARACTER*8 CHTAG(NBIG)
      CHARACTER*8 CHPARS(21)

C... VEQ for reading ntuples raw into it
      REAL VEQ(NBIG)
      INTEGER NV

C...used in  HGIVEN to check actual param numbers in ntuple
      CHARACTER*80 CTIT

      NEVRD=0
      LRECL=1024

      CALL HROPEN( LUNHB, 'ABC', DSNHB , ' ', LRECL, ISTAT )

C... get Ntuple into PAWC
      CALL HRIN (NTID,9999,0)

C... print header.
      IF (IDEBUG.EQ.-7) CALL HPRNTU(NTID)

C... Read variable names => CHTAG(..),
C... and actual nuber of parameters => NV
      NV=NBIG
      CALL HGIVEN (NTID,CTIT,NV,CHTAG, RLOW, RHIGH)

      IF (IERR .NE. 0) THEN
         PRINT*,'===> NTROWR: ERROR in HGIVEN'
      ENDIF

C... copy those which are used to vector CHNAMS, used by HGNTV
      DO 2 J = 1,NV
c... this magic 21 is NBANM in ani.f; programming on fortran
c... without common blocks is smthg :(
         IF (J.GT.21) GOTO 2
         CHPARS(J) = CHTAG(J)
2     CONTINUE

C... Check the validness of parameters
      IF (NV.LT.NPARAC) THEN
         PRINT*,'==> NTROWR: Number of parameters requested too large!'
         PRINT*,'             Only ',NV,' parameters in ntuple',NTID,' '
         PRINT*,'             from file ',DSNHB
         GO TO 30
      ENDIF

      DO 15 I=1,NV
       IF (NUMBEV(I).GT.NV) THEN
       PRINT*,'==> NTROWR: Parameter ',NUMBEV(I),' requested too large!'
       PRINT*,'             Only ',NV,' parameters in ntuple',NTID,' '
       PRINT*,'             from file ',DSNHB
       GO TO 30
      ENDIF
15    CONTINUE

C... Get number of events in Ntuple
      CALL HNOENT(NTID,NOENT)
      IF (NEVTAC.GT.NOENT) THEN
         NEVTAC = NOENT
         PRINT*,'==> NTROWR: Number of events requested too large!'
         PRINT*,'             Only ',NOENT,' events are present!'
      ENDIF

C----------------------------------------------------------------------
C...  HGNF uses this
      CALL HGNPAR (NTID,' ntrowr')

C... Loop over events, read parameters
      NEV = 0

      DO 10 I = 1,NEVTAC
         IF (LEVTOR) THEN
C... read according to user-defined sequence
            J = EVTORI(I)
         ELSE
C... just read them in incremental order scipping the first
C... NEVTSK ones
            J = I + NEVTSK
         ENDIF
C... check whether j is valid event number
         IF (J.GT.NOENT) THEN
            PRINT*,'==> ERROR: requested event ',J,' does not exist!'
            GOTO 10
         ENDIF
C... actual Ntuple read-routine =>  VEQ(1),...
         CALL HGNF(NTID, J, VEQ(1), IERR)
         IF (IERR .NE. 0) THEN
            PRINT*,'===> NTROWR: ERROR in HGNF, reading event # ',J
         ENDIF
C... cuts on parameters
         LCUT = .FALSE.
         DO 5 J = 1,NPARAC
C... set flag LCUT if cuts apply
            IF (VEQ(NUMBEV(J)).LT.PARCTL(J).OR.
     +          VEQ(NUMBEV(J)).GT.PARCTU(J)) LCUT = .TRUE.
C... to avoid double looping, copy event regardless, in case
C... of cut this entry will be overwritten
            EVENTS(J,NEV+1) = VEQ(NUMBEV(J))
C... set all weights equal; it will be normed to 1 later in ANI
            EVTWT(NEV+1)    = 1.
   5     CONTINUE

         IF (IDEBUG.EQ.-7) THEN
             write(*,*) (EVENTS(ii,NEV+1),ii=1,NPARAC)
         ENDIF

C... if cuts didn't apply, store event number and increment
C... event counter
         IF (.NOT.LCUT) THEN
            NEV = NEV + 1
            EVTORO(NEV) = J
         ENDIF
  10  CONTINUE
C----------------------------------------------------------------------
c
C... in case cuts applied to the LAST event, delete this entry
      IF (LCUT) THEN
         DO 20 J = 1,NPARAC
            EVENTS(J,NEV+1) = 0.0
            EVTWT(NEV+1)    = 0.0
  20     CONTINUE
      ENDIF
c
C... events actually read <- event counter
      NEVRD = NEV

C... leave here if there were errors
30    CONTINUE

C... normal HBOOK closing procedure
      CALL HREND(LUNHB)

C... The end.
      RETURN
      END
