*CMZ :          01/02/96  13.58.14  by  Eugene H. Sevinian
*-- Author :
C*DK TIE
C=======================================================================
C
C     ..................................................................
C
C        SUBROUTINE TIE
C
C        PURPOSE
C           CALCULATE CORRECTION FACTOR DUE TO TIES
C
C        USAGE
C           CALL TIE(R,N,KT,T)
C
C        DESCRIPTION OF PARAMETERS
C           R  - INPUT VECTOR OF RANKS OF LENGTH N CONTAINING VALUES
C                1 TO N
C           N  - NUMBER OF RANKED VALUES
C           KT - INPUT CODE FOR CALCULATION OF CORRECTION FACTOR
C                      1   SOLVE EQUATION 1
C                      2   SOLVE EQUATION 2
C           T  - CORRECTION FACTOR (OUTPUT)
C                    EQUATION 1   T=SUM(CT**3-CT)/12
C                    EQUATION 2   T=SUM(CT*(CT-1)/2)
C                  WHERE CT IS THE NUMBER OF OBSERVATIONS TIED FOR A
C                        GIVEN RANK
C
C        REMARKS
C           NONE
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           VECTOR IS SEARCHED FOR SUCCESSIVELY LARGER RANKS. TIES ARE
C           COUNTED AND CORRECTION FACTOR 1 OR 2 SUMMED.
C
C     ..................................................................
C
      SUBROUTINE TIE(R,N,KT,T)
      DIMENSION R(1)
C
C        INITIALIZATION
C
      T=0.0
      Y=0.0
   10 X=1.0E38
      IND=0
C
C        FIND NEXT LARGEST RANK
C
      DO 40 I=1,N
         IF(R(I)-Y) 40,40,20
   20    IF(R(I)-X) 30,40,40
   30    X=R(I)
         IND=IND+1
   40 CONTINUE
C
C        IF ALL RANKS HAVE BEEN TESTED, RETURN
C
      IF(IND) 110,110,50
   50 Y=X
      CT=0.0
C
C        COUNT TIES
C
      DO 70 I=1,N
         IF(R(I)-X) 70,60,70
   60    CT=CT+1.0
   70 CONTINUE
C
C        CALCULATE CORRECTION FACTOR
C
      IF(CT) 80,10,80
   80 IF(KT-1) 90,100,90
   90 T=T+CT*(CT-1.)/2.0
      GO TO 10
  100 T=T+(CT*CT*CT-CT)/12.0
      GO TO 10
  110 RETURN
      END
