*CMZ :          20/06/97  18.16.41  by  Ararat Vardanyan
*-- Author :    Mark Halpaap   11/05/94
C-----------------------------------------------------------------------
C NTREAD reads a HBOOK-Ntuple. Cuts may be applied on each parameter.
C It is possible to skip events or to read them in a selected order.
C A subset of parameters may be chosen. Those events that were chosen
C to be read and survived the cuts will be returned, including the
C corresponding weights and the order after which they were read.

      SUBROUTINE NTREAD(NTID,LUNHB,LUNMSG,DEBUG,DSNHB,NPARMX,NEVTMX,
     +                NPARAC,NEVTAC,NEVRD,NEVTSK,LEVTOR,NUMBEV,CHPARS,
     +                EVTORI,EVTORO,PARCTL,PARCTU,EVTWT,EVENTS)

* in the following * and C indicate INPUT and OUTPUT, respectively.

*...NTID  : Identifier (ID) of the Ntuple to be read
*...LUNHB : HBOOK - Unit
*...LUNMSG: Unit for messages according to verbose-level DEBUG
*...DEBUG : verbose-level (0...5)
*...DSNHB : HBOOK - Filename
*...NPARMX: Maximum number of parameters
*...NEVTMX: Maximum number of events
*...NPARAC: Actual number of parameters (to be used)
*...NEVTAC: Actual number of events (to be read)
C...NEVRD : Nr of events actually read
*...NEVTSK: Number of events to be skipped
*...LEVTOR: FLAG: read events in selected order? (=> .TRUE.)
*...NUMBEV: Vector of parameter-numbers to be used
C...CHPARS: CHAR*8-Vector of corresponding parameter-descriptors (names)
*...EVTORI: Vector containing Event ORder to use if LEVTOR==.TRUE.
C...EVTORO: Vector containing Event ORder AFTER reading (and maybe cuts)
*...PARCTL: Vector containing LOWER cuts on parameters
*...PARCTU: Vector containing UPPER cuts on parameters
C...EVTWT : Vector containing event WEIGHTS
C...EVENTS: Array containing the events in terms of the chosen parameters


      DIMENSION EVENTS(NPARAC,NEVTMX), EVTWT(NEVTMX)
      DIMENSION PARCTL(NPARMX), PARCTU(NPARMX)
      DIMENSION EVTORI(NEVTMX), EVTORO(NEVTMX)
      DIMENSION NUMBEV(NPARMX)
      LOGICAL   LCUT, LEVTOR
      CHARACTER*50 DSNHB

C... CWN need COMMONs to communicate
      REAL          V1,V2,V3,V4,V5,V6,V7,V8,V9,V10,V11,V12,V13,V14,V15,
     +              V16,V17,V18,V19,V20,V21
      COMMON /V/    V1,V2,V3,V4,V5,V6,V7,V8,V9,V10,V11,V12,V13,V14,V15,
     +              V16,V17,V18,V19,V20,V21
C... to fully exploit CWNs, booking is best NOT done with arrays.
C... However, arrays are MUCH more convenient, therefore:
      REAL VEQ(21)
      EQUIVALENCE (VEQ(1),V1)
C... into CH the variable names are loaded, into CHNAMS only those
C... which are used are copied. This enables selective Ntuple reading.
      CHARACTER*8 CH(21)
      CHARACTER*8 CHPARS(21)
      CHARACTER*8 CHNAMS(21)
      COMMON /SHOCHA/ CH

C... more stable than HROPEN
      OPEN (UNIT=LUNHB,FILE=DSNHB,ACCESS='DIRECT',RECL=1024,
     +      STATUS='UNKNOWN')
      CALL HRFILE (LUNHB,'NT',' ')

C... get Ntuple into PAWC, print header.
C      CALL HRIN (NTID,9999,0)
C      CALL HRIN (NTID+1,9999,0)
      CALL HRIN (0,9999,0)
      CALL HPRNT(NTID)

C... The $CLEAR - directive is needed to prepare each Ntuple-ID,
C... the $SET - directive is needed to prepare each 'BLOCK'
C... therein.
      CALL HBNAME (NTID,' ',0.,'$CLEAR')
      CALL HBNAME (NTID,'PARS',V1,'$SET')
C... the second Ntuple contains only the variable names
      CALL HBNAME (NTID+1,' ',0.,'$CLEAR')
      CALL HBNAMC (NTID+1,'CHARS',CH,'$SET')

C... Read variable names => CH(..)
      CALL HGNT (NTID+1,1,IERR)
      IF (IERR .NE. 0) THEN
         PRINT*,'===> NTREAD: ERROR in HGNT, reading variable names'
      ENDIF

C... copy those which are used to vector CHNAMS, used by HGNTV
      DO 2 J = 1,NPARAC
         CHNAMS(J) = CH(NUMBEV(J))
   2  CONTINUE
C... take care that weights are read
      CHNAMS(NPARAC+1) = CH(21)

C... Get nr of events in Ntuple
      CALL HNOENT(NTID,NOENT)
      IF (NEVTAC.GT.NOENT) THEN
         NEVTAC = NOENT
         PRINT*,'==> WARNING: Number of events requested too large!'
         PRINT*,'             Only ',NOENT,' events are present!'
      ENDIF

C... read first event, give info according to DEBUG, prepare for
C... later use of HGNTF, which is much faster
      CALL HGNTV(NTID,CHNAMS,NPARAC+1,1,IERR)
      IF (IERR .NE. 0) THEN
         PRINT*,'===> NTREAD: ERROR in HGNTV, reading 1st event'
      ENDIF
C... debug printout and copying of variable names
      DO k=1,21
        CHPARS(k)=CH(K)
        print*,veq(k)
      enddo

C----------------------------------------------------------------------
C... Loop over events, read parameters
      NEV = 0
      DO 10 I = 1,NEVTAC
         IF (LEVTOR) THEN
C... read according to user-defined sequence
            J = EVTORI(I)
         ELSE
C... just read them in incremental order, skipping the first
C... NEVTSK ones
            J = I + NEVTSK
         ENDIF
C... check whether j is valid event number
         IF (J.GT.NOENT) THEN
            PRINT*,'==> ERROR: requested event ',J,' does not exist!'
            GOTO 10
         ENDIF
C... actual Ntuple read-routine => V1,... EQUIV VEQ(1),...
         CALL HGNTF(NTID,J,IERR)
         IF (IERR .NE. 0) THEN
            PRINT*,'===> NTREAD: ERROR in HGNTF, reading event # ',J
         ENDIF
C... cuts on parameters
         LCUT = .FALSE.
         DO 5 J = 1,NPARAC
C... set flag LCUT if cuts apply
            IF (VEQ(NUMBEV(J)).LT.PARCTL(J).OR.
     +          VEQ(NUMBEV(J)).GT.PARCTU(J)) LCUT = .TRUE.
C... to avoid double looping, copy event regardless, in case
C... of cut this entry will be overwritten
            EVENTS(J,NEV+1) = VEQ(NUMBEV(J))
            EVTWT(NEV+1)    = VEQ(21)
   5     CONTINUE
C... if cuts didn't apply, store event number and increment
C... event counter
         IF (.NOT.LCUT) THEN
            NEV = NEV + 1
            EVTORO(NEV) = J
         ENDIF
  10  CONTINUE
C----------------------------------------------------------------------
c
C... in case cuts applied to the LAST event, delete this entry
      IF (LCUT) THEN
         DO 20 J = 1,NPARAC
            EVENTS(J,NEV+1) = 0.0
            EVTWT(NEV+1)    = 0.0
  20     CONTINUE
      ENDIF
c
C... events actually read <- event counter
      NEVRD = NEV

C... normal HBOOK closing procedure
      CALL HREND('NT')
      CLOSE(LUNHB)
c
C... The end.
      RETURN
      END
